import struct

from pycoin.encoding.hexbytes import bytes_as_revhex
from pycoin.serialize.streamer import Streamer

from .satoshi_int import parse_satoshi_int, stream_satoshi_int
from .satoshi_string import parse_satoshi_string, stream_satoshi_string


STREAMER_FUNCTIONS = {
    "I": (parse_satoshi_int, stream_satoshi_int),
    "S": (parse_satoshi_string, stream_satoshi_string),
    "h": (lambda f: struct.unpack("!H", f.read(2))[0], lambda f, v: f.write(struct.pack("!H", v))),
    "L": (lambda f: struct.unpack("<L", f.read(4))[0], lambda f, v: f.write(struct.pack("<L", v))),
    "Q": (lambda f: struct.unpack("<Q", f.read(8))[0], lambda f, v: f.write(struct.pack("<Q", v))),
    "#": (lambda f: bytes_as_revhex(f.read(32)), lambda f, v: f.write(v[:32])),
    "@": (lambda f: f.read(16), lambda f, v: f.write(v[:16])),
    "b": (lambda f: struct.unpack("?", f.read(1))[0], lambda f, b: f.write(struct.pack("?", b))),
}

SATOSHI_STREAMER = Streamer()
SATOSHI_STREAMER.register_array_count_parse(parse_satoshi_int)
SATOSHI_STREAMER.register_functions(STREAMER_FUNCTIONS.items())



import os
import shlex
import sys
import tempfile
import unittest

try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO

from pycoin.cmds import block, ku, msg, tx, coinc


DEFAULT_ENV = {
    "PYCOIN_BTC_PROVIDERS": "blockchain.info blockexplorer.com",
    "PATH": os.environ.get("PATH")
}


TOOL_LOOKUP = {
    "tx": (tx.create_parser(), tx.tx),
    "ku": (ku.create_parser(), ku.ku),
    "msg": (msg.create_parser(), msg.msg),
    "block": (block.create_parser(), block.block),
    "coinc": (coinc.create_parser(), coinc.coinc),
}


class ToolTest(unittest.TestCase):

    def invoke_tool(self, args):
        tool_name = args[0]
        parser, main = TOOL_LOOKUP[tool_name]
        return main(parser.parse_args(args[1:]), parser)

    def launch_tool(self, cmd_line=None, args=None, env=None):
        if args is None:
            args = shlex.split(cmd_line)

        new_environ = dict(env or {})
        new_environ.update(DEFAULT_ENV)

        # capture io
        new_stdout = StringIO()
        old_stdout = sys.stdout
        old_environ = os.environ

        os.environ = new_environ
        sys.stdout = new_stdout

        self.invoke_tool(args)

        sys.stdout = old_stdout
        os.environ = old_environ
        output = new_stdout.getvalue()
        return output

    def set_cache_dir(self):
        temp_dir = tempfile.mkdtemp()
        return temp_dir




import decimal

SATOSHI_PER_COIN = decimal.Decimal(int(1e8))
COIN_PER_SATOSHI = decimal.Decimal(1)/SATOSHI_PER_COIN

SATOSHI_TO_MBTC = decimal.Decimal(int(1e5))
MBTC_PER_SATOSHI = 1/SATOSHI_TO_MBTC


def satoshi_to_btc(satoshi_count):
    if satoshi_count == 0:
        return decimal.Decimal(0)
    r = satoshi_count * COIN_PER_SATOSHI
    return r.quantize(COIN_PER_SATOSHI)


def btc_to_satoshi(btc):
    return int(decimal.Decimal(btc) * SATOSHI_PER_COIN)


def satoshi_to_mbtc(satoshi_count):
    if satoshi_count == 0:
        return decimal.Decimal(0)
    r = satoshi_count / SATOSHI_TO_MBTC
    return r.quantize(MBTC_PER_SATOSHI)


def mbtc_to_satoshi(btc):
    return int(decimal.Decimal(btc) * SATOSHI_TO_MBTC)



from pycoin.networks.bitcoinish import create_bitcoinish_network

network = create_bitcoinish_network(
    symbol="DCRT", network_name="Decred", subnet_name="testnet",
    wif_prefix_hex="230e", address_prefix_hex="0f21", pay_to_script_prefix_hex="0e6c",
    bip32_prv_prefix_hex="04358397", bip32_pub_prefix_hex="043587d1")




class NoSuchPointError(ValueError):
    pass


class Point(tuple):
    """
    A point on an elliptic curve. This is a subclass of tuple (forced to a 2-tuple),
    and also includes a reference to the underlying Curve.

    This class supports the operators ``+``, ``-`` (unary and binary) and ``*``.

    :param x: x coordinate
    :param y: y coordinate
    :param curve: the :class:`Curve <pycoin.ecdsa.Curve.Curve>` this point must be on

    The constructor raises :class:`NoSuchPointError` if the point is invalid.
    The point at infinity is ``(x, y) == (None, None)``.
    """
    def __new__(self, x, y, curve):
        """
        Subclasses of tuple require __new__ to be overridden.
        """
        return tuple.__new__(self, (x, y))

    def __init__(self, x, y, curve):
        self._curve = curve
        super(Point, self).__init__()
        self.check_on_curve()

    def check_on_curve(self):
        """raise :class:`NoSuchPointError` if the point is not actually on the curve."""
        if not self._curve.contains_point(*self):
            raise NoSuchPointError('({},{}) is not on the curve {}'.format(self[0], self[1], self._curve))

    def __add__(self, other):
        """Add one point to another point."""
        return self._curve.add(self, other)

    def __sub__(self, other):
        """Subtract one point from another point."""
        return self._curve.add(self, -other)

    def __mul__(self, e):
        """Multiply a point by an integer."""
        return self._curve.multiply(self, e)

    def __rmul__(self, other):
        """Multiply a point by an integer."""
        return self * other

    def __neg__(self):
        """Unary negation"""
        return self.__class__(self[0], self._curve.p()-self[1], self._curve)

    def curve(self):
        """:return: the :class:`Curve <pycoin.ecdsa.Curve>` this point is on"""
        return self._curve



from ..bitcoin.Solver import BitcoinSolver
from pycoin.satoshi.flags import SIGHASH_ALL, SIGHASH_FORKID

from .SolutionChecker import BgoldSolutionChecker


class BgoldSolver(BitcoinSolver):
    SolutionChecker = BgoldSolutionChecker

    def solve(self, *args, **kwargs):
        if kwargs.get("hash_type") is None:
            kwargs["hash_type"] = SIGHASH_ALL
        kwargs["hash_type"] |= SIGHASH_FORKID
        return super(BgoldSolver, self).solve(*args, **kwargs)



import unittest

from pycoin.symbols.btc import network


# BRAIN DAMAGE
who_signed_tx = network.who_signed.who_signed_tx
Tx = network.tx
SIGHASH_ALL = network.validator.flags.SIGHASH_ALL


class WhoSignedTest(unittest.TestCase):

    def multisig_M_of_N(self, M, N, unsigned_id, signed_id):
        keys = [network.keys.private(secret_exponent=i) for i in range(1, N+2)]
        tx_in = Tx.TxIn.coinbase_tx_in(script=b'')
        script = network.contract.for_multisig(m=M, sec_keys=[key.sec() for key in keys[:N]])
        tx_out = Tx.TxOut(1000000, script)
        tx1 = Tx(version=1, txs_in=[tx_in], txs_out=[tx_out])
        tx2 = network.tx_utils.create_tx(tx1.tx_outs_as_spendable(), [keys[-1].address()])
        self.assertEqual(tx2.id(), unsigned_id)
        self.assertEqual(tx2.bad_solution_count(), 1)
        hash160_lookup = network.tx.solve.build_hash160_lookup((key.secret_exponent() for key in keys[:M]))
        tx2.sign(hash160_lookup=hash160_lookup)
        self.assertEqual(tx2.id(), signed_id)
        self.assertEqual(tx2.bad_solution_count(), 0)
        self.assertEqual(sorted(who_signed_tx(tx2, 0)),
                         sorted(((key.address(), SIGHASH_ALL) for key in keys[:M])))

    def test_create_multisig_1_of_2(self):
        unsigned_id = "dd40f601e801ad87701b04851a4a6852d6b625e481d0fc9c3302faf613a4fc88"
        signed_id = "fb9ccc00d0e30ab2648768104fd777df8f856830233232c5e43f43584aec23d9"
        self.multisig_M_of_N(1, 2, unsigned_id, signed_id)

    def test_create_multisig_2_of_3(self):
        unsigned_id = "6bc5614a41c7c4aa828f5a4314fff23e5e49b1137e5d31e9716eb58f6fb198ff"
        signed_id = "c521962fe9d0e5efb7d0966759c57e7ee2595ce8e05cb342b19265a8722420dd"
        self.multisig_M_of_N(2, 3, unsigned_id, signed_id)

    def test_multisig_one_at_a_time(self):
        M = 3
        N = 3
        keys = [network.keys.private(secret_exponent=i) for i in range(1, N+2)]
        tx_in = Tx.TxIn.coinbase_tx_in(script=b'')
        script = network.contract.for_multisig(m=M, sec_keys=[key.sec() for key in keys[:N]])
        tx_out = Tx.TxOut(1000000, script)
        tx1 = Tx(version=1, txs_in=[tx_in], txs_out=[tx_out])
        tx2 = network.tx_utils.create_tx(tx1.tx_outs_as_spendable(), [keys[-1].address()])
        ids = ["403e5bfc59e097bb197bf77a692d158dd3a4f7affb4a1fa41072dafe7bec7058",
               "5931d9995e83721243dca24772d7012afcd4378996a8b953c458175f15a544db",
               "9bb4421088190bbbb5b42a9eaa9baed7ec7574a407c25f71992ba56ca43d9c44",
               "03a1dc2a63f93a5cf5a7cb668658eb3fc2eda88c06dc287b85ba3e6aff751771"]
        for i in range(1, N+1):
            self.assertEqual(tx2.bad_solution_count(), 1)
            self.assertEqual(tx2.id(), ids[i-1])
            hash160_lookup = network.tx.solve.build_hash160_lookup([keys[i-1].secret_exponent()])
            tx2.sign(hash160_lookup=hash160_lookup)
            self.assertEqual(tx2.id(), ids[i])
            t1 = sorted(who_signed_tx(tx2, 0))
            t2 = sorted(((key.address(), SIGHASH_ALL) for key in keys[:i]))
            self.assertEqual(t1, t2)
        self.assertEqual(tx2.bad_solution_count(), 0)

    def test_sign_pay_to_script_multisig(self):
        M, N = 3, 3
        keys = [network.keys.private(secret_exponent=i) for i in range(1, N+2)]
        tx_in = Tx.TxIn.coinbase_tx_in(script=b'')
        underlying_script = network.contract.for_multisig(m=M, sec_keys=[key.sec() for key in keys[:N]])
        address = network.address.for_p2s(underlying_script)
        self.assertEqual(address, "39qEwuwyb2cAX38MFtrNzvq3KV9hSNov3q")
        script = network.contract.for_address(address)
        tx_out = Tx.TxOut(1000000, script)
        tx1 = Tx(version=1, txs_in=[tx_in], txs_out=[tx_out])
        tx2 = network.tx_utils.create_tx(tx1.tx_outs_as_spendable(), [address])
        hash160_lookup = network.tx.solve.build_hash160_lookup((key.secret_exponent() for key in keys[:N]))
        p2sh_lookup = network.tx.solve.build_p2sh_lookup([underlying_script])
        tx2.sign(hash160_lookup=hash160_lookup, p2sh_lookup=p2sh_lookup)
        self.assertEqual(tx2.bad_solution_count(), 0)
        self.assertEqual(sorted(who_signed_tx(tx2, 0)),
                         sorted(((key.address(), SIGHASH_ALL) for key in keys[:M])))


if __name__ == "__main__":
    unittest.main()



from .constraints import Atom, Operator


class SolvingError(Exception):
    pass


class CONSTANT(object):
    def __init__(self, name):
        self._name = name

    def match(self, c):
        if not isinstance(c, Atom):
            return {self._name: c}
        return False


class VAR(object):
    def __init__(self, name):
        self._name = name

    def match(self, c):
        if isinstance(c, Atom) and not isinstance(c, Operator):
            return {self._name: c}
        return False


class LIST(object):
    def __init__(self, name):
        self._name = name

    def match(self, c):
        if isinstance(c, (tuple, list)):
            return {self._name: c}
        return False


class ConstraintSolver(object):
    def __init__(self):
        self._solvers_for_patterns = {}

    def register_solver(self, pattern, solver_f):
        self._solvers_for_patterns[pattern] = solver_f

    def solutions_for_constraint(self, c):
        # given a constraint c
        # return None or
        # a solution (solution_f, target atom, dependency atom list)
        # where solution_f take list of solved values

        for pattern, f_factory in self._solvers_for_patterns.items():
            m = self.constraint_matches(c, pattern)
            if m:
                return f_factory(m)

    def constraint_matches(self, c, m):
        """
        Return dict noting the substitution values (or False for no match)
        """
        if isinstance(m, tuple):
            d = {}
            if isinstance(c, Operator) and c._op_name == m[0]:
                for c1, m1 in zip(c._args, m[1:]):
                    r = self.constraint_matches(c1, m1)
                    if r is False:
                        return r
                    d.update(r)
                return d
            return False
        return m.match(c)



import unittest

from pycoin.cmds.tx import DEFAULT_VERSION
from pycoin.encoding.hexbytes import h2b
from pycoin.symbols.btc import network


Spendable = network.tx.Spendable
Tx = network.tx
TxIn = network.tx.TxIn
TxOut = network.tx.TxOut


class SignTest(unittest.TestCase):

    def test_sign_p2sh(self):
        tx_out_script = h2b("76a91491b24bf9f5288532960ac687abb035127b1d28a588ac")
        script = network.contract.for_address("1EHNa6Q4Jz2uvNExL497mE43ikXhwF6kZm")
        self.assertEqual(tx_out_script, script)
        tx_out = TxOut(100, tx_out_script)
        tx = Tx(1, [TxIn(b'\1' * 32, 1)], [TxOut(100, tx_out_script)])
        tx.set_unspents([tx_out])
        hl = network.tx.solve.build_hash160_lookup([1])
        self.assertEqual(tx.bad_solution_count(), 1)
        tx.sign(hash160_lookup=hl)
        self.assertEqual(tx.bad_solution_count(), 0)

    def multisig_M_of_N(self, M, N, unsigned_id, signed_id):
        keys = [network.keys.private(secret_exponent=i) for i in range(1, N+2)]
        tx_in = TxIn.coinbase_tx_in(script=b'')
        script = network.contract.for_multisig(m=M, sec_keys=[key.sec() for key in keys[:N]])
        tx_out = TxOut(1000000, script)
        tx1 = Tx(version=1, txs_in=[tx_in], txs_out=[tx_out])
        tx2 = network.tx_utils.create_tx(tx1.tx_outs_as_spendable(), [keys[-1].address()])
        self.assertEqual(tx2.id(), unsigned_id)
        self.assertEqual(tx2.bad_solution_count(), 1)
        hash160_lookup = network.tx.solve.build_hash160_lookup((key.secret_exponent() for key in keys[:M]))
        tx2.sign(hash160_lookup=hash160_lookup)
        self.assertEqual(tx2.id(), signed_id)
        self.assertEqual(tx2.bad_solution_count(), 0)

    def test_sign_multisig_1_of_2(self):
        unsigned_id = "dd40f601e801ad87701b04851a4a6852d6b625e481d0fc9c3302faf613a4fc88"
        signed_id = "fb9ccc00d0e30ab2648768104fd777df8f856830233232c5e43f43584aec23d9"
        self.multisig_M_of_N(1, 2, unsigned_id, signed_id)

    def test_sign_multisig_2_of_3(self):
        unsigned_id = "6bc5614a41c7c4aa828f5a4314fff23e5e49b1137e5d31e9716eb58f6fb198ff"
        signed_id = "c521962fe9d0e5efb7d0966759c57e7ee2595ce8e05cb342b19265a8722420dd"
        self.multisig_M_of_N(2, 3, unsigned_id, signed_id)

    def test_multisig_one_at_a_time(self):
        M = 3
        N = 3
        keys = [network.keys.private(secret_exponent=i) for i in range(1, N+2)]
        tx_in = TxIn.coinbase_tx_in(script=b'')
        script = network.contract.for_multisig(m=M, sec_keys=[key.sec() for key in keys[:N]])
        tx_out = TxOut(1000000, script)
        tx1 = Tx(version=1, txs_in=[tx_in], txs_out=[tx_out])
        tx2 = network.tx_utils.create_tx(tx1.tx_outs_as_spendable(), [keys[-1].address()])
        ids = ["403e5bfc59e097bb197bf77a692d158dd3a4f7affb4a1fa41072dafe7bec7058",
               "5931d9995e83721243dca24772d7012afcd4378996a8b953c458175f15a544db",
               "9bb4421088190bbbb5b42a9eaa9baed7ec7574a407c25f71992ba56ca43d9c44",
               "03a1dc2a63f93a5cf5a7cb668658eb3fc2eda88c06dc287b85ba3e6aff751771"]
        for i in range(1, N+1):
            self.assertEqual(tx2.bad_solution_count(), 1)
            self.assertEqual(tx2.id(), ids[i-1])
            hash160_lookup = network.tx.solve.build_hash160_lookup((key.secret_exponent() for key in keys[i-1:i]))
            tx2.sign(hash160_lookup=hash160_lookup)
            self.assertEqual(tx2.id(), ids[i])
        self.assertEqual(tx2.bad_solution_count(), 0)

    def test_p2sh_multisig_sequential_signing(self):
        raw_scripts = [h2b(
            "52210234abcffd2e80ad01c2ec0276ad02682808169c6fafdd25ebfb60703df272b461"
            "2102e5baaafff8094e4d77ce8b009d5ebc3de9110085ebd3d96e50cc7ce70faf175221"
            "0316ee25e80eb6e6fc734d9c86fa580cbb9c4bfd94a19f0373a22353ececd4db6853ae")]
        spendable = {'script_hex': 'a914c4ed4de526461e3efbb79c8b688a6f9282c0464687', 'does_seem_spent': 0,
                     'block_index_spent': 0, 'coin_value': 10000, 'block_index_available': 0, 'tx_out_index': 0,
                     'tx_hash_hex': '0ca152ba6b88db87a7ef1afd24554102aca1ab86cf2c10ccbc374472145dc943'}

        key_1 = network.parse.wif('Kz6pytJCigYHeMsGLmfHQPJhN5og2wpeSVrU43xWwgHLCAvpsprh')
        key_2 = network.parse.wif('Kz7NHgX7MBySA3RSKj9GexUSN6NepEDoPNugSPr5absRDoKgn2dT')
        for ordered_keys in [(key_1, key_2), (key_2, key_1)]:
            txs_in = [TxIn(previous_hash=h2b('43c95d14724437bccc102ccf86aba1ac02415524fd1aefa787db886bba52a10c'),
                           previous_index=0)]
            txs_out = [TxOut(10000, network.contract.for_address('3KeGeLFmsbmbVdeMLrWp7WYKcA3tdsB4AR'))]
            unspents = [Spendable.from_dict(spendable)]
            tx = Tx(version=DEFAULT_VERSION, txs_in=txs_in, txs_out=txs_out, unspents=unspents)
            for key in ordered_keys:
                self.assertEqual(tx.bad_solution_count(), 1)
                p2sh_lookup = network.tx.solve.build_p2sh_lookup(raw_scripts)
                tx.sign(network.tx.solve.build_hash160_lookup([key.secret_exponent()]), p2sh_lookup=p2sh_lookup)
            self.assertEqual(tx.bad_solution_count(), 0)

    def test_sign_pay_to_script_multisig(self):
        M, N = 3, 3
        keys = [network.keys.private(secret_exponent=i) for i in range(1, N+2)]
        tx_in = TxIn.coinbase_tx_in(script=b'')
        underlying_script = network.contract.for_multisig(m=M, sec_keys=[key.sec() for key in keys[:N]])
        address = network.address.for_p2s(underlying_script)
        self.assertEqual(address, "39qEwuwyb2cAX38MFtrNzvq3KV9hSNov3q")
        script = network.contract.for_address(address)
        tx_out = TxOut(1000000, script)
        tx1 = Tx(version=1, txs_in=[tx_in], txs_out=[tx_out])
        tx2 = network.tx_utils.create_tx(tx1.tx_outs_as_spendable(), [address])
        hash160_lookup = network.tx.solve.build_hash160_lookup((key.secret_exponent() for key in keys[:N]))
        p2sh_lookup = network.tx.solve.build_p2sh_lookup([underlying_script])
        tx2.sign(hash160_lookup=hash160_lookup, p2sh_lookup=p2sh_lookup)
        self.assertEqual(tx2.bad_solution_count(), 0)

    def test_sign_bitcoind_partially_signed_2_of_2(self):
        # Finish signing a 2 of 2 transaction, that already has one signature signed by bitcoind
        # This tx can be found on testnet3 blockchain
        # txid: 9618820d7037d2f32db798c92665231cd4599326f5bd99cb59d0b723be2a13a2
        raw_script = ("522103e33b41f5ed67a77d4c4c54b3e946bd30d15b8f66e42cb29fde059c168851165521"
                      "02b92cb20a9fb1eb9656a74eeb7387636cf64cdf502ff50511830328c1b479986452ae")
        p2sh_lookup = network.tx.solve.build_p2sh_lookup([h2b(raw_script)])
        partially_signed_raw_tx = (
            "010000000196238f11a5fd3ceef4efd5a186a7e6b9217d900418e72aca917cd6a6e634"
            "e74100000000910047304402201b41b471d9dd93cf97eed7cfc39a5767a546f6bfbf3e"
            "0c91ff9ad23ab9770f1f02205ce565666271d055be1f25a7e52e34cbf659f6c70770ff"
            "59bd783a6fcd1be3dd0147522103e33b41f5ed67a77d4c4c54b3e946bd30d15b8f66e4"
            "2cb29fde059c16885116552102b92cb20a9fb1eb9656a74eeb7387636cf64cdf502ff5"
            "0511830328c1b479986452aeffffffff01a0bb0d00000000001976a9143b3beefd6f78"
            "02fa8706983a76a51467bfa36f8b88ac00000000")
        tx = Tx.from_hex(partially_signed_raw_tx)
        tx_out = TxOut(1000000, h2b("a914a10dfa21ee8c33b028b92562f6fe04e60563d3c087"))
        tx.set_unspents([tx_out])
        key = network.parse.wif("L3LRiWuBJ6xcY4rnUsEiEX8mXRmKZG7uYyJ2sePDxSZ2haw1hPHW")
        hash160_lookup = network.tx.solve.build_hash160_lookup([key.secret_exponent()])
        self.assertEqual(tx.bad_solution_count(), 1)
        tx.sign(hash160_lookup=hash160_lookup, p2sh_lookup=p2sh_lookup)
        self.assertEqual(tx.bad_solution_count(), 0)
        self.assertEqual(tx.id(), "9618820d7037d2f32db798c92665231cd4599326f5bd99cb59d0b723be2a13a2")


if __name__ == "__main__":
    unittest.main()




class ScriptError(Exception):
    def error_code(self):
        if len(self.args) > 1:
            return self.args[1]
        return None


class SolutionChecker(object):

    ScriptError = ScriptError

    def __init__(self, *args, **kwargs):
        raise

    def check_solution(self, tx_context, traceback_f=None, *args, **kwargs):
        """
        tx_context: information about the transaction that the VM may need
        traceback_f: a function invoked on occasion to check intermediate state
        """
        raise NotImplemented()



import io

from pycoin.coins.bitcoin.Tx import Tx as BaseTx
from pycoin.convention import SATOSHI_PER_COIN
from pycoin.satoshi.satoshi_struct import stream_struct

from .hash import sha256
from .Solver import GroestlcoinSolver as Solver
from .SolutionChecker import GroestlcoinSolutionChecker as SolutionChecker


class Tx(BaseTx):
    SolutionChecker = SolutionChecker
    Solver = Solver

    MAX_MONEY = 105000000 * SATOSHI_PER_COIN

    def hash(self, hash_type=None):
        s = io.BytesIO()
        self.stream(s, include_witness_data=False)
        if hash_type is not None:
            stream_struct("L", s, hash_type)
        return sha256(s.getvalue())

    def w_hash(self):
        return sha256(self.as_bin())

    def blanked_hash(self):
        s = io.BytesIO()
        self.stream(s, blank_solutions=True)
        return sha256(s.getvalue())



import struct

from pycoin.coins.SolutionChecker import ScriptError
from pycoin.satoshi import errno, opcodes
from pycoin.satoshi.IntStreamer import IntStreamer
from pycoin.vm.ScriptStreamer import ScriptStreamer


def make_opcode_const_list():
    return [("OP_%d" % i, IntStreamer.int_to_script_bytes(i)) for i in range(17)] + [
            ("OP_1NEGATE", IntStreamer.int_to_script_bytes(-1))]


def make_opcode_sized_list():
    return [("OP_PUSH_%d" % i, i) for i in range(1, 76)]


def make_opcode_variable_list():
    def make_variable_decoder(struct_data):
        struct_size = struct.calcsize(struct_data)

        def decode_OP_PUSHDATA(script, pc):
            pc += 1
            try:
                size = struct.unpack(struct_data, script[pc:pc+struct_size])[0]
            except Exception:
                return 0, pc
            pc += struct_size
            return size, pc
        return decode_OP_PUSHDATA

    OPCODE_VARIABLE_LIST = [
        ("OP_PUSHDATA1", (1 << 8)-1, lambda d: struct.pack("<B", d), make_variable_decoder("<B")),
        ("OP_PUSHDATA2", (1 << 16)-1, lambda d: struct.pack("<H", d), make_variable_decoder("<H")),
        ("OP_PUSHDATA4", (1 << 32)-1, lambda d: struct.pack("<L", d), make_variable_decoder("<L"))
    ]
    return OPCODE_VARIABLE_LIST


def non_minimal_f(msg):
    raise ScriptError(msg, errno.MINIMALDATA)


def make_script_streamer():
    OPCODE_CONST_LIST = make_opcode_const_list()
    OPCODE_SIZED_LIST = make_opcode_sized_list()
    OPCODE_VARIABLE_LIST = make_opcode_variable_list()
    OPCODE_LOOKUP = dict(o for o in opcodes.OPCODE_LIST)

    return ScriptStreamer(
        OPCODE_CONST_LIST, OPCODE_SIZED_LIST, OPCODE_VARIABLE_LIST, OPCODE_LOOKUP, non_minimal_f)


BitcoinScriptStreamer = make_script_streamer()



from pycoin.networks.ParseAPI import ParseAPI
from pycoin.networks.parseable_str import parseable_str, parse_b58

from .hash import groestlHash


def b58_groestl(s):
    data = parse_b58(s)
    if data:
        data, the_hash = data[:-4], data[-4:]
        if groestlHash(data)[:4] == the_hash:
            return data


def parse_b58_groestl(s):
    s = parseable_str(s)
    return s.cache("b58_groestl", b58_groestl)


class GRSParseAPI(ParseAPI):
    """Set GRS parse functions."""

    def parse_b58_hashed(self, s):
        return parse_b58_groestl(s)



import collections
import hashlib

from pycoin.encoding.hash import hash160
from pycoin.encoding.hexbytes import b2h


class ContractAPI(object):
    def __init__(self, network, script_tools):
        self._network = network
        self._script_tools = script_tools

    def for_address(self, address):
        info = self._network.parse.address(address)
        if info:
            return info.script()

    def for_p2pk(self, sec):
        return self.for_info(dict(type="p2pk", sec=sec))

    def for_p2pkh(self, hash160):
        return self.for_info(dict(type="p2pkh", hash160=hash160))

    def for_p2pkh_wit(self, hash160):
        return self.for_info(dict(type="p2pkh_wit", hash160=hash160))

    def for_p2sh(self, hash160):
        return self.for_info(dict(type="p2sh", hash160=hash160))

    def for_p2sh_wit(self, hash256):
        return self.for_info(dict(type="p2sh_wit", hash256=hash256))

    def for_multisig(self, m, sec_keys):
        return self.for_info(dict(type="multisig", m=m, sec_keys=sec_keys))

    def for_nulldata(self, data):
        return self.for_info(dict(type="nulldata", data=data))

    def for_nulldata_push(self, data):
        # BRAIN DAMAGE
        return self._script_tools.compile("OP_RETURN [%s]" % b2h(data))

    # BRAIN DAMAGE: the stuff above is redundant

    def for_p2s(self, underlying_script):
        return self.for_p2sh(hash160(underlying_script))

    def for_p2s_wit(self, underlying_script):
        return self.for_p2sh_wit(hashlib.sha256(underlying_script).digest())

    def match(self, template_disassembly, script):
        template = self._script_tools.compile(template_disassembly)
        r = collections.defaultdict(list)
        pc1 = pc2 = 0
        while 1:
            if pc1 == len(script) and pc2 == len(template):
                return r
            if pc1 >= len(script) or pc2 >= len(template):
                break
            opcode1, data1, pc1, is_ok2 = self._script_tools.scriptStreamer.get_opcode(script, pc1)
            opcode2, data2, pc2, is_ok2 = self._script_tools.scriptStreamer.get_opcode(template, pc2)
            l1 = 0 if data1 is None else len(data1)
            if data2 == b'PUBKEY':
                if l1 < 33 or l1 > 120:
                    break
                r["PUBKEY_LIST"].append(data1)
            elif data2 == b'PUBKEYHASH':
                if l1 != 160/8:
                    break
                r["PUBKEYHASH_LIST"].append(data1)
            elif data2 == b'SEGWIT':
                if l1 not in (256/8, 160/8):
                    break
                r["SEGWIT_LIST"].append(data1)
            elif data2 == b'DATA':
                r["DATA_LIST"].append(data1)
            elif (opcode1, data1) != (opcode2, data2):
                break
        return None

    _SCRIPT_LOOKUP = dict(
        p2pk=lambda info: "%s OP_CHECKSIG" % b2h(info.get("sec")),
        p2pkh=lambda info: "OP_DUP OP_HASH160 %s OP_EQUALVERIFY OP_CHECKSIG" % b2h(info.get("hash160")),
        p2pkh_wit=lambda info: "OP_0 %s" % b2h(info.get("hash160")),
        p2sh=lambda info: "OP_HASH160 %s OP_EQUAL" % b2h(info.get("hash160")),
        p2sh_wit=lambda info: "OP_0 %s" % b2h(info.get("hash256")),
        multisig=lambda info: "%d %s %d OP_CHECKMULTISIG" % (
            info.get("m"), " ".join(b2h(sk) for sk in info.get("sec_keys")), len(info.get("sec_keys"))),
    )

    def for_info(self, info):
        type = info.get("type")
        if type == "nulldata":
            return self._script_tools.compile("OP_RETURN") + info.get("data")
        if type == "unknown":
            return info["script"]
        script_text = self._SCRIPT_LOOKUP[type](info)
        return self._script_tools.compile(script_text)

    # MISSING to consider
    # p2s: SCRIPTHASH160
    # nulldata_push: DATA, RAW_DATA

    def info_for_script(self, script):
        d = self.match("OP_DUP OP_HASH160 'PUBKEYHASH' OP_EQUALVERIFY OP_CHECKSIG", script)
        if d:
            return dict(type="p2pkh", hash160=d["PUBKEYHASH_LIST"][0])

        d = self.match("OP_0 'SEGWIT'", script)
        if d:
            data = d["SEGWIT_LIST"][0]
            if len(data) == 20:
                return dict(type="p2pkh_wit", hash160=data)
            if len(data) == 32:
                return dict(type="p2sh_wit", hash256=data)

        d = self.match("OP_HASH160 'PUBKEYHASH' OP_EQUAL", script)
        if d:
            return dict(type="p2sh", hash160=d["PUBKEYHASH_LIST"][0])

        d = self.match("'PUBKEY' OP_CHECKSIG", script)
        if d:
            return dict(type="p2pk", sec=d["PUBKEY_LIST"][0])

        if self._script_tools.compile("OP_RETURN") == script[:1]:
            return dict(type="nulldata", data=script[1:])

        d = self._info_from_multisig_script(script)
        if d:
            return d

        return dict(type="unknown", script=script)

    def _info_from_multisig_script(self, script):
        script_tools = self._script_tools
        scriptStreamer = script_tools.scriptStreamer
        OP_1 = script_tools.int_for_opcode("OP_1")
        OP_16 = script_tools.int_for_opcode("OP_16")
        pc = 0
        if len(script) == 0:
            return None
        opcode, data, pc, is_ok = scriptStreamer.get_opcode(script, pc)

        if not OP_1 <= opcode < OP_16:
            return None
        m = opcode + (1 - OP_1)
        sec_keys = []
        while pc < len(script):
            opcode, data, pc, is_ok = scriptStreamer.get_opcode(script, pc)
            size = len(data) if data else 0
            if size < 33 or size > 120:
                break
            sec_keys.append(data)
        if pc >= len(script):
            return None
        n = opcode + (1 - OP_1)
        if m > n or len(sec_keys) != n:
            return None

        opcode, data, pc, is_ok = scriptStreamer.get_opcode(script, pc)
        OP_CHECKMULTISIG = script_tools.int_for_opcode("OP_CHECKMULTISIG")
        if opcode != OP_CHECKMULTISIG:
            return None
        if pc != len(script):
            return None
        return dict(type="multisig", sec_keys=sec_keys, m=m)



# Adapted from code written in 2005 by Peter Pearson and placed in the public domain.


from .Point import Point


def _leftmost_bit(x):
    # this is closer to constant time than bit-twiddling hacks like those in
    # https://graphics.stanford.edu/~seander/bithacks.html
    assert x > 0
    result = 1
    while result <= x:
        result <<= 1
    return result >> 1


class Curve(object):
    """
    This class implements an `Elliptic curve <https://en.wikipedia.org/wiki/Elliptic_curve>`_ intended
    for use in `Elliptic curve cryptography <https://en.wikipedia.org/wiki/Elliptic-curve_cryptography>`_

    An elliptic curve ``EC<p, a, b>`` for a (usually large) prime p and integers a and b is a
    `group <https://en.wikipedia.org/wiki/Group_(mathematics)>`_. The members of the group are
    (x, y) points (where x and y are integers over the field of integers modulo p) that satisfy the relation
    ``y**2 = x**3 + a*x + b (mod p)``. There is a group operation ``+`` and an extra point known
    as the "point at infinity" thrown in to act as the identity for the group.

    The group operation is a truly marvelous property of this construct, a description of which
    this margin is too narrow to contain, so please refer to the links above for more information.

    :param p: a prime
    :param a: an integer coefficient
    :param b: an integer constant
    :param order: (optional) the order of the group made up by the points on the
        curve. Any point on the curve times the order is the identity for this
        group (the point at infinity). Although this is optional, it's required
        for some operations.
    """
    def __init__(self, p, a, b, order=None):
        """
        """
        self._p = p
        self._a = a
        self._b = b
        self._order = order
        self._infinity = Point(None, None, self)

    def p(self):
        """
        :returns: the prime modulus of the curve.
        """
        return self._p

    def order(self):
        """
        :returns: the order of the curve.
        """
        return self._order

    def infinity(self):
        """:returns: the "point at infinity" (also known as 0, or the identity)."""
        return self._infinity

    def contains_point(self, x, y):
        """
        :param x: x coordinate of a point
        :param y: y coordinate of a point
        :returns: True if the point (x, y) is on the curve, False otherwise
        """
        if x is None and y is None:
            return True
        return (y * y - (x * x * x + self._a * x + self._b)) % self._p == 0

    def add(self, p0, p1):
        """
        :param p0: a point
        :param p1: a point
        :returns: the sum of the two points
        """
        p = self._p
        infinity = self._infinity

        if p0 == infinity:
            return p1
        if p1 == infinity:
            return p0

        x0, y0 = p0
        x1, y1 = p1
        if (x0 - x1) % p == 0:
            if (y0 + y1) % p == 0:
                return infinity
            else:
                slope = ((3 * x0 * x0 + self._a) * self.inverse_mod(2 * y0, p)) % p
        else:
            slope = ((y1 - y0) * self.inverse_mod(x1 - x0, p)) % p

        x3 = (slope * slope - x0 - x1) % p
        y3 = (slope * (x0 - x3) - y0) % p

        return self.Point(x3, y3)

    def multiply(self, p, e):
        """
        multiply a point by an integer.

        :param p: a point
        :param e: an integer
        :returns: the result, equivalent to adding p to itself e times
        """

        if self._order:
            e %= self._order
        if p == self._infinity or e == 0:
            return self._infinity

        e3 = 3 * e
        i = _leftmost_bit(e3) >> 1
        result = p
        while i > 1:
            result += result
            if (e3 & i):
                v = [result, result+p]
            else:
                v = [result-p, result]
            result = v[0 if (e & i) else 1]
            i >>= 1

        return result

    def inverse_mod(self, a, m):
        """
        :param a: an integer
        :param m: another integer
        :returns: the value ``b`` such that ``a * b == 1 (mod m)``
        """

        if a < 0 or m <= a:
            a = a % m

        # From Ferguson and Schneier, roughly:

        c, d = a, m
        uc, vc, ud, vd = 1, 0, 0, 1
        while c != 0:
            q, c, d = divmod(d, c) + (c,)
            uc, vc, ud, vd = ud - q*uc, vd - q*vc, uc, vc

        # At this point, d is the GCD, and ud*a+vd*m = d.
        # If d == 1, this means that ud is a inverse.

        assert d == 1
        if ud > 0:
            return ud
        else:
            return ud + m

    def Point(self, x, y):
        """
        :returns: a :class:`Point <.Point>` object with coordinates ``(x, y)``
        """
        return Point(x, y, self)

    def __repr__(self):
        return '{}({!r},{!r},{!r})'.format(self.__class__.__name__, self._p, self._a, self._b)

    def __str__(self):
        return 'y^2 = x^3 + {}*x + {} (mod {})'.format(self._a, self._b, self._p)



#!/usr/bin/env python

# This script shows you how to spend coins from an incoming transaction.
# It expects an incoming transaction in hex format a file ("incoming-tx.hex")
# and a bitcoin address, and it spends the coins from the selected output of
# in incoming transaction to the address you choose.

# It does NOT sign the transaction. That's done by 4_sign_tx.py.

import sys

from pycoin.symbols.btc import network


def main():
    if len(sys.argv) != 4:
        print("usage: %s incoming_tx_hex_filename tx_out_index new_address" % sys.argv[0])
        sys.exit(-1)

    with open(sys.argv[1], "r") as f:
        tx_hex = f.readline().strip()

    # get the spendable from the prior transaction
    tx = network.tx.from_hex(tx_hex)
    tx_out_index = int(sys.argv[2])
    spendable = tx.tx_outs_as_spendable()[tx_out_index]

    # make sure the address is valid
    payable = sys.argv[3]
    assert network.parse.address(payable) is not None

    # create the unsigned transaction
    tx = network.tx_utils.create_tx([spendable], [payable])

    print("here is the transaction: %s" % tx.as_hex(include_unspents=True))


if __name__ == '__main__':
    main()



from pycoin.networks.bitcoinish import create_bitcoinish_network

network = create_bitcoinish_network(
    symbol="DASH", network_name="Dash", subnet_name="mainnet",
    wif_prefix_hex="cc", address_prefix_hex="4c", pay_to_script_prefix_hex="10",
    bip32_prv_prefix_hex="02fe52f8", bip32_pub_prefix_hex="02fe52cc")



import io

from pycoin.block import Block as BaseBlock

from .hash import groestlHash
from .Tx import Tx


class Block(BaseBlock):
    Tx = Tx

    def _calculate_hash(self):
        s = io.BytesIO()
        self.stream_header(s)
        return groestlHash(s.getvalue())



import unittest

from pycoin.symbols.btc import network


class ElectrumTest(unittest.TestCase):
    def test_initial_key(self):
        RECEIVING_ADDRESSES = [
            "1LDkC1H438qSnJLHCYkQ3WTZQkSEwoYGHc",
            "12mENAcc8ZhZbR6hv7LGm3jV7PwbYeF8Xk",
            "1A3NpABFd6YHvwr1ti1r8brU3BzQuV2Nr4",
            "1Gn6nWAoZrpmtV9zuNbyivWvRBpcygWaQX",
            "1M5i5P3DhtDbnvSTfmnUbcrTVgF8GDWQW9"
        ]
        CHANGE_ADDRESSES = [
            "1iiAbyBTh1J69UzD1JcrfW8JSVJ9ve9gT",
            "146wnqmsQNYCZ6AXRCqLkzZyGM1ZU6nr3F",
            "1Mwexajvia3s8AcaGUkyEg9ZZJPJeTbKTZ"
        ]
        wallet = network.keys.electrum_seed(seed="00000000000000000000000000000001")
        for idx, address in enumerate(RECEIVING_ADDRESSES):
            subkey = wallet.subkey("%s/0" % idx)
            calculated_address = subkey.address()
            self.assertEqual(address, calculated_address)
            wif = subkey.wif()
            key = network.parse.wif(wif)
            self.assertEqual(key.address(), address)
        for idx, address in enumerate(CHANGE_ADDRESSES):
            subkey = wallet.subkey("%s/1" % idx)
            calculated_address = subkey.address()
            self.assertEqual(address, calculated_address)
            wif = subkey.wif()
            key = network.parse.wif(wif)
            self.assertEqual(key.address(), address)

    def test_master_public_and_private(self):
        # these addresses were generated by hand using electrum with a master public key
        # corresponding to secret exponent 1
        RECEIVING_ADDRESSES = [
            "1AYPdHLna6bKFUbeXoAEVbaXUxifUwCMay",
            "13UeuWJba5epizAKyfCfiFKY5Kbxfdxe7B",
            "19f6KJUTL5AGBRvLBGiL6Zpcx53QA7zaKT",
            "1Cm33VuSkoUETwx5nsF1wgmGqYwJZxpZdY",
            "14Z6ErkETixQMUeivsYbrdoUFns2J1iSct",
        ]
        CHANGE_ADDRESSES = [
            "1JVYsmjrqSy1BKvo1gYpNjX7AYea74nQYe",
            "1Cc7itfQaDqZK3vHYphFsySujQjBNba8mw",
            "15wrXvrAnyv3usGeQRohnnZ8tz9XAekbag",
            "1MnWCEjE5YiZpZrkP8HcXEeDqwg43RxLwu",
            "1Fgyp3PUx9AAg8yJe1zGXHP5dVC6i1tXbs",
            "12XTLd4u9jeqw4egLAUhoKLxHARCdKWkty"
        ]
        k = network.keys.private(secret_exponent=1)
        master_public_key = k.sec(is_compressed=False)[1:]

        wallet = network.keys.electrum_public(master_public_key)
        for idx, address in enumerate(RECEIVING_ADDRESSES):
            subkey = wallet.subkey("%s/0" % idx)
            calculated_address = subkey.address()
            self.assertEqual(address, calculated_address)
        for idx, address in enumerate(CHANGE_ADDRESSES):
            subkey = wallet.subkey("%s/1" % idx)
            calculated_address = subkey.address()
            self.assertEqual(address, calculated_address)

        wallet = network.keys.electrum_private(master_private_key=1)
        for idx, address in enumerate(RECEIVING_ADDRESSES):
            subkey = wallet.subkey("%s/0" % idx)
            calculated_address = subkey.address()
            self.assertEqual(address, calculated_address)
            wif = subkey.wif()
            key = network.parse.wif(wif)
            self.assertEqual(key.address(), address)
        for idx, address in enumerate(CHANGE_ADDRESSES):
            subkey = wallet.subkey("%s/1" % idx)
            calculated_address = subkey.address()
            self.assertEqual(address, calculated_address)
            wif = subkey.wif()
            key = network.parse.wif(wif)
            self.assertEqual(key.address(), address)



from .Key import Key
from .subpaths import subpaths_for_path_range


class HierarchicalKey(Key):
    def subkeys(self, path):
        """
        A generalized form that can return multiple subkeys.
        """
        for _ in subpaths_for_path_range(path, hardening_chars="'pH"):
            yield self.subkey_for_path(_)

    def output(self):
        yield ("wallet_key", self.hwif(as_private=self.is_private()), None)
        if self.is_private():
            yield ("public_version", self.hwif(as_private=False), None)

        child_number = self.child_index()
        if child_number >= 0x80000000:
            wc = child_number - 0x80000000
            child_index = "%dH (%d)" % (wc, child_number)
        else:
            child_index = "%d" % child_number
        yield ("tree_depth", "%d" % self.tree_depth(), None)
        yield ("fingerprint", b2h(self.fingerprint()), None)
        yield ("parent_fingerprint", b2h(self.parent_fingerprint()), "parent f'print")
        yield ("child_index", child_index, None)
        yield ("chain_code", b2h(self.chain_code()), None)

        yield ("private_key", "yes" if self.is_private() else "no", None)



from pycoin.networks.bitcoinish import create_bitcoinish_network


network = create_bitcoinish_network(
    symbol="XRT", network_name="Bitcoin", subnet_name="regtest",
    wif_prefix_hex="ef", sec_prefix="XRTSEC:", address_prefix_hex="6f", pay_to_script_prefix_hex="c4",
    bip32_prv_prefix_hex="04358394", bip32_pub_prefix_hex="043587CF", bech32_hrp="bcrt",
    magic_header_hex="0B110907", default_port=18444)



from pycoin.networks.bitcoinish import create_bitcoinish_network


network = create_bitcoinish_network(
    network_name="Litecoin", symbol="XLT", subnet_name="testnet",
    wif_prefix_hex="ef", sec_prefix="XLTSEC:", address_prefix_hex="6f", pay_to_script_prefix_hex="c4",
    bip32_prv_prefix_hex="0436ef7d", bip32_pub_prefix_hex="0436f6e1", bech32_hrp="tltc")



import io
import unittest

from pycoin.encoding.hexbytes import h2b
from pycoin.symbols.btc import network


Block = network.block
Tx = network.tx
flags = network.validator.flags

# block 80971
block_80971_cs = h2b('00000000001126456C67A1F5F0FF0268F53B4F22E0531DC70C7B69746AF69DAC')
block_80971_data = h2b(
    "01000000950A1631FB9FAC411DFB173487B9E18018B7C6F7147E78C062584100000000"
    "00A881352F97F14BF191B54915AE124E051B8FE6C3922C5082B34EAD503000FC34D891"
    "974CED66471B4016850A04010000000100000000000000000000000000000000000000"
    "00000000000000000000000000FFFFFFFF0804ED66471B02C301FFFFFFFF0100F2052A"
    "01000000434104CB6B6B4EADC96C7D08B21B29D0ADA5F29F9378978CABDB602B8B65DA"
    "08C8A93CAAB46F5ABD59889BAC704925942DD77A2116D10E0274CAD944C71D3D1A6705"
    "70AC0000000001000000018C55ED829F16A4E43902940D3D33005264606D5F7D555B5F"
    "67EE4C033390C2EB010000008A47304402202D1BF606648EDCDB124C1254930852D991"
    "88E1231715031CBEAEA80CCFD2B39A02201FA9D6EE7A1763580E342474FC1AEF59B046"
    "8F98479953437F525063E25675DE014104A01F763CFBF5E518C628939158AF3DC0CAAC"
    "35C4BA7BC1CE8B7E634E8CDC44E15F0296B250282BD649BAA8398D199F2424FCDCD88D"
    "3A9ED186E4FD3CB9BF57CFFFFFFFFF02404B4C00000000001976A9148156FF75BEF24B"
    "35ACCE3C05289A2411E1B0E57988AC00AA38DF010000001976A914BC7E692A5FFE95A5"
    "96712F5ED83393B3002E452E88AC0000000001000000019C97AFDF6C9A31FFA86D71EA"
    "79A079001E2B59EE408FD418498219400639AC0A010000008B4830450220363CFFAE09"
    "599397B21E6D8A8073FB1DFBE06B6ACDD0F2F7D3FEA86CA9C3F605022100FA255A6ED2"
    "3FD825C759EF1A885A31CAD0989606CA8A3A16657D50FE3CEF5828014104FF444BAC08"
    "308B9EC97F56A652AD8866E0BA804DA97868909999566CB377F4A2C8F1000E83B49686"
    "8F3A282E1A34DF78565B65C15C3FA21A0763FD81A3DFBBB6FFFFFFFF02C05EECDE0100"
    "00001976A914588554E6CC64E7343D77117DA7E01357A6111B7988AC404B4C00000000"
    "001976A914CA6EB218592F289999F13916EE32829AD587DBC588AC0000000001000000"
    "01BEF5C9225CB9FE3DEF929423FA36AAD9980B9D6F8F3070001ACF3A5FB389A69F0000"
    "00004A493046022100FB23B1E2F2FB8B96E04D220D385346290A9349F89BBBC5C225D5"
    "A56D931F8A8E022100F298EB28294B90C1BAF319DAB713E7CA721AAADD8FCC15F849DE"
    "7B0A6CF5412101FFFFFFFF0100F2052A010000001976A9146DDEA8071439951115469D"
    "0D2E2B80ECBCDD48DB88AC00000000")

block_80971 = Block.parse(io.BytesIO(block_80971_data))

COINBASE_PUB_KEY_FROM_80971 = h2b(
    "04cb6b6b4eadc96c7d08b21b29d0ada5f29f9378978cabdb602b8b65da08c8a93caab46"
    "f5abd59889bac704925942dd77a2116d10e0274cad944c71d3d1a670570")
COINBASE_BYTES_FROM_80971 = h2b("04ed66471b02c301")


def standard_tx(coins_from, coins_to):
    txs_in = []
    unspents = []
    for h, idx, tx_out in coins_from:
        txs_in.append(Tx.TxIn(h, idx))
        unspents.append(tx_out)

    txs_out = []
    for coin_value, address in coins_to:
        txs_out.append(Tx.TxOut(coin_value, network.contract.for_address(address)))

    version, lock_time = 1, 0
    tx = Tx(version, txs_in, txs_out, lock_time)
    tx.set_unspents(unspents)
    return tx


class BuildTxTest(unittest.TestCase):

    def test_signature_hash(self):
        compressed = False
        exponent_2 = int("137f3276686959c82b454eea6eefc9ab1b9e45bd4636fb9320262e114e321da1", 16)
        address_2 = network.keys.private(exponent_2, is_compressed=compressed).address()
        key = network.parse.wif("5JMys7YfK72cRVTrbwkq5paxU7vgkMypB55KyXEtN5uSnjV7K8Y")
        exponent = key.secret_exponent()

        public_key_sec = network.keys.private(exponent, is_compressed=compressed).sec()

        the_coinbase_tx = Tx.coinbase_tx(public_key_sec, int(50 * 1e8), COINBASE_BYTES_FROM_80971)
        coins_from = [(the_coinbase_tx.hash(), 0, the_coinbase_tx.txs_out[0])]
        coins_to = [(int(50 * 1e8), address_2)]
        unsigned_coinbase_spend_tx = standard_tx(coins_from, coins_to)

        tx_out_script_to_check = the_coinbase_tx.txs_out[0].script
        idx = 0
        solution_checker = Tx.SolutionChecker(unsigned_coinbase_spend_tx)
        actual_hash = solution_checker._signature_hash(tx_out_script_to_check, idx, hash_type=flags.SIGHASH_ALL)
        self.assertEqual(actual_hash, 29819170155392455064899446505816569230970401928540834591675173488544269166940)

    def test_standard_tx_out(self):
        coin_value = 10
        recipient_address = '1BcJRKjiwYQ3f37FQSpTYM7AfnXurMjezu'
        tx_out = standard_tx([], [(coin_value, recipient_address)]).txs_out[0]
        s = str(tx_out)
        self.assertEqual(
            s,
            'TxOut<0.00010 mbtc "OP_DUP OP_HASH160 [745e5b81fd30ca1e90311b012badabaa4411ae1a]'
            ' OP_EQUALVERIFY OP_CHECKSIG">')

    def test_coinbase_tx(self):
        tx = Tx.coinbase_tx(COINBASE_PUB_KEY_FROM_80971, int(50 * 1e8), COINBASE_BYTES_FROM_80971)
        s = io.BytesIO()
        tx.stream(s)
        tx1 = s.getvalue()
        s = io.BytesIO()
        block_80971.txs[0].stream(s)
        tx2 = s.getvalue()
        self.assertEqual(tx1, tx2)

    def test_tx_out_address(self):
        tx = Tx.coinbase_tx(COINBASE_PUB_KEY_FROM_80971, int(50 * 1e8), COINBASE_BYTES_FROM_80971)
        address = network.address.for_script(tx.txs_out[0].puzzle_script())
        self.assertEqual(address, '1DmapcnrJNGeJB13fv9ngRFX1iRvR4zamn')

    def test_build_spends(self):
        # first, here is the tx database
        TX_DB = {}

        # create a coinbase Tx where we know the public & private key

        key = network.parse.wif("5JMys7YfK72cRVTrbwkq5paxU7vgkMypB55KyXEtN5uSnjV7K8Y")
        exponent = key.secret_exponent()
        compressed = False

        public_key_sec = network.keys.private(exponent, is_compressed=compressed).sec()

        the_coinbase_tx = Tx.coinbase_tx(public_key_sec, int(50 * 1e8), COINBASE_BYTES_FROM_80971)
        TX_DB[the_coinbase_tx.hash()] = the_coinbase_tx

        # now create a Tx that spends the coinbase

        compressed = False

        exponent_2 = int("137f3276686959c82b454eea6eefc9ab1b9e45bd4636fb9320262e114e321da1", 16)
        address_2 = network.keys.private(exponent_2, is_compressed=compressed).address()

        self.assertEqual("12WivmEn8AUth6x6U8HuJuXHaJzDw3gHNZ", address_2)

        coins_from = [(the_coinbase_tx.hash(), 0, the_coinbase_tx.txs_out[0])]
        coins_to = [(int(50 * 1e8), address_2)]
        unsigned_coinbase_spend_tx = standard_tx(coins_from, coins_to)
        solver = network.tx.solve.build_hash160_lookup([exponent])

        coinbase_spend_tx = unsigned_coinbase_spend_tx.sign(solver)

        TX_DB[coinbase_spend_tx.hash()] = coinbase_spend_tx

        # now try to respend from priv_key_2 to priv_key_3

        compressed = True

        exponent_3 = int("f8d39b8ecd0e1b6fee5a340519f239097569d7a403a50bb14fb2f04eff8db0ff", 16)
        address_3 = network.keys.private(exponent_3, is_compressed=compressed).address()

        self.assertEqual("13zzEHPCH2WUZJzANymow3ZrxcZ8iFBrY5", address_3)

        coins_from = [(coinbase_spend_tx.hash(), 0, coinbase_spend_tx.txs_out[0])]
        unsigned_spend_tx = standard_tx(coins_from, [(int(50 * 1e8), address_3)])
        solver.update(network.tx.solve.build_hash160_lookup([exponent_2]))
        spend_tx = unsigned_spend_tx.sign(solver)

        # now check that it validates
        self.assertEqual(spend_tx.bad_solution_count(), 0)


if __name__ == '__main__':
    unittest.main()



from pycoin.networks.bitcoinish import create_bitcoinish_network


network = create_bitcoinish_network(
    symbol="POLIS", network_name="Polis", subnet_name="mainnet",
    wif_prefix_hex="3C", address_prefix_hex="37", pay_to_script_prefix_hex="3C",
    bip32_prv_prefix_hex="03e25d7e", bip32_pub_prefix_hex="03e25945",
    magic_header_hex="BD6B0CBF", default_port=24126,
    dns_bootstrap=[
        "dnsseed.poliscentral.org", "dnsseed2.poliscentral.org",
        "dnsseed3.poliscentral.org", "polis.seeds.mn.zone", "polis.mnseeds.com"
    ])



from .providers import (spendables_for_address, get_tx_db,
                        get_default_providers_for_netcode, set_default_providers_for_netcode)

__all__ = ['spendables_for_address', 'get_tx_db', 'get_default_providers_for_netcode',
           'set_default_providers_for_netcode']



from pkg_resources import get_distribution, DistributionNotFound

try:
    version = get_distribution(__name__).version
except DistributionNotFound:
    # package is not installed
    version = "unknown"

__title__ = 'pycoin'
__author__ = 'Richard Kiss'
__version__ = version
__license__ = 'MIT'
__copyright__ = 'Copyright 2018 Richard Kiss'

"""
:copyright: (c) 2018 by Richard Kiss
:license: MIT, see LICENSE for more details.
"""



import math
import struct

from pycoin.encoding.b58 import a2b_hashed_base58
from pycoin.intbytes import indexbytes

LOG_2 = math.log(2)


def filter_size_required(element_count, false_positive_probability):
    # The size S of the filter in bytes is given by
    # (-1 / pow(log(2), 2) * N * log(P)) / 8
    # Of course you must ensure it does not go over the maximum size
    # (36,000: selected as it represents a filter of 20,000 items with false
    # positive rate of < 0.1% or 10,000 items and a false positive rate of < 0.0001%).
    lfpp = math.log(false_positive_probability)
    return min(36000, int(((-1 / pow(LOG_2, 2) * element_count * lfpp)+7) // 8))


def hash_function_count_required(filter_size, element_count):
    # The number of hash functions required is given by S * 8 / N * log(2).
    return int(filter_size * 8.0 / element_count * LOG_2 + 0.5)


class BloomFilter(object):
    MASK_ARRAY = [1 << _ for _ in range(8)]

    def __init__(self, size_in_bytes, hash_function_count, tweak):
        if size_in_bytes > 36000:
            raise ValueError("too large")
        self.filter_bytes = bytearray(size_in_bytes)
        self.bit_count = 8 * size_in_bytes
        self.hash_function_count = hash_function_count
        self.tweak = tweak

    def add_item(self, item_bytes):
        for hash_index in range(self.hash_function_count):
            seed = hash_index * 0xFBA4C795 + self.tweak
            self.set_bit(murmur3(item_bytes, seed=seed) % self.bit_count)

    def add_address(self, address):
        the_hash160 = a2b_hashed_base58(address)[1:]
        self.add_item(the_hash160)

    def add_hash160(self, the_hash160):
        self.add_item(the_hash160)

    def add_spendable(self, spendable):
        item_bytes = spendable.tx_hash + struct.pack("<L", spendable.tx_out_index)
        self.add_item(item_bytes)

    def _index_for_bit(self, v):
        v %= self.bit_count
        byte_index, mask_index = divmod(v, 8)
        mask = self.MASK_ARRAY[mask_index]
        return byte_index, mask

    def set_bit(self, v):
        byte_index, mask = self._index_for_bit(v)
        self.filter_bytes[byte_index] |= mask

    def check_bit(self, v):
        byte_index, mask = self._index_for_bit(v)
        return (self.filter_bytes[byte_index] & mask) == mask

    def filter_load_params(self):
        return self.filter_bytes, self.hash_function_count, self.tweak


# http://stackoverflow.com/questions/13305290/is-there-a-pure-python-implementation-of-murmurhash

def murmur3(data, seed=0):
    c1 = 0xcc9e2d51
    c2 = 0x1b873593

    length = len(data)
    h1 = seed
    roundedEnd = (length & 0xfffffffc)  # round down to 4 byte block
    for i in range(0, roundedEnd, 4):
        # little endian load order
        k1 = (indexbytes(data, i) & 0xff) | ((indexbytes(data, i + 1) & 0xff) << 8) | \
            ((indexbytes(data, i + 2) & 0xff) << 16) | (indexbytes(data, i + 3) << 24)
        k1 *= c1
        k1 = (k1 << 15) | ((k1 & 0xffffffff) >> 17)  # ROTL32(k1,15)
        k1 *= c2

        h1 ^= k1
        h1 = (h1 << 13) | ((h1 & 0xffffffff) >> 19)  # ROTL32(h1,13)
        h1 = h1 * 5 + 0xe6546b64

    # tail
    k1 = 0

    val = length & 0x03
    if val == 3:
        k1 = (indexbytes(data, roundedEnd + 2) & 0xff) << 16
    # fallthrough
    if val in [2, 3]:
        k1 |= (indexbytes(data, roundedEnd + 1) & 0xff) << 8
    # fallthrough
    if val in [1, 2, 3]:
        k1 |= indexbytes(data, roundedEnd) & 0xff
        k1 *= c1
        k1 = (k1 << 15) | ((k1 & 0xffffffff) >> 17)  # ROTL32(k1,15)
        k1 *= c2
        h1 ^= k1

    # finalization
    h1 ^= length

    # fmix(h1)
    h1 ^= ((h1 & 0xffffffff) >> 16)
    h1 *= 0x85ebca6b
    h1 ^= ((h1 & 0xffffffff) >> 13)
    h1 *= 0xc2b2ae35
    h1 ^= ((h1 & 0xffffffff) >> 16)

    return h1 & 0xffffffff



import unittest

from pycoin.encoding.hexbytes import h2b
from pycoin.symbols.btc import network


class KeyTest(unittest.TestCase):

    def test_sign_verify(self):
        private_key = network.keys.private(secret_exponent=1)
        h = b"\x16" * 32
        sig = private_key.sign(h)
        self.assertTrue(private_key.verify(h, sig))
        public_key = private_key.public_copy()
        self.assertTrue(public_key.verify(h, sig))

    def test_translation(self):
        def do_test(exp_hex, wif, c_wif, public_pair_sec, c_public_pair_sec, address_b58, c_address_b58):
            secret_exponent = int(exp_hex, 16)
            sec = h2b(public_pair_sec)
            c_sec = h2b(c_public_pair_sec)

            keys_wif = [
                network.keys.private(secret_exponent=secret_exponent),
                network.parse.wif(wif),
                network.parse.wif(c_wif),
            ]

            key_sec = network.keys.public(sec)
            key_sec_c = network.keys.public(c_sec)
            keys_sec = [key_sec, key_sec_c]

            for key in keys_wif:
                self.assertEqual(key.secret_exponent(), secret_exponent)
                self.assertEqual(key.public_copy().secret_exponent(), None)
                repr(key)
                if key.is_compressed():
                    self.assertEqual(key.wif(), c_wif)
                self.assertEqual(key.wif(is_compressed=False), wif)
                self.assertEqual(key.wif(is_compressed=True), c_wif)

            for key in keys_wif + keys_sec:
                if key.is_compressed():
                    self.assertEqual(key.sec(), c_sec)
                self.assertEqual(key.sec(is_compressed=False), sec)
                self.assertEqual(key.sec(is_compressed=True), c_sec)
                if key.is_compressed():
                    self.assertEqual(key.address(), c_address_b58)
                else:
                    self.assertEqual(key.address(), address_b58)
                self.assertEqual(key.address(is_compressed=True), c_address_b58)
                self.assertEqual(key.address(is_compressed=False), address_b58)

            key_pub = network.parse.address(address_b58)

            self.assertEqual(key_pub.address(), address_b58)

        do_test("1111111111111111111111111111111111111111111111111111111111111111",
                "5HwoXVkHoRM8sL2KmNRS217n1g8mPPBomrY7yehCuXC1115WWsh",
                "KwntMbt59tTsj8xqpqYqRRWufyjGunvhSyeMo3NTYpFYzZbXJ5Hp",
                "044f355bdcb7cc0af728ef3cceb9615d90684bb5b2ca5f859ab0f0b704075871aa"
                "385b6b1b8ead809ca67454d9683fcf2ba03456d6fe2c4abe2b07f0fbdbb2f1c1",
                "034f355bdcb7cc0af728ef3cceb9615d90684bb5b2ca5f859ab0f0b704075871aa",
                "1MsHWS1BnwMc3tLE8G35UXsS58fKipzB7a",
                "1Q1pE5vPGEEMqRcVRMbtBK842Y6Pzo6nK9")

        do_test("dddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddd",
                "5KVzsHJiUxgvBBgtVS7qBTbbYZpwWM4WQNCCyNSiuFCJzYMxg8H",
                "L4ezQvyC6QoBhxB4GVs9fAPhUKtbaXYUn8YTqoeXwbevQq4U92vN",
                "04ed83704c95d829046f1ac27806211132102c34e9ac7ffa1b71110658e5b9d1bd"
                "edc416f5cefc1db0625cd0c75de8192d2b592d7e3b00bcfb4a0e860d880fd1fc",
                "02ed83704c95d829046f1ac27806211132102c34e9ac7ffa1b71110658e5b9d1bd",
                "1JyMKvPHkrCQd8jQrqTR1rBsAd1VpRhTiE",
                "1NKRhS7iYUGTaAfaR5z8BueAJesqaTyc4a")

        do_test("47f7616ea6f9b923076625b4488115de1ef1187f760e65f89eb6f4f7ff04b012",
                "5JMys7YfK72cRVTrbwkq5paxU7vgkMypB55KyXEtN5uSnjV7K8Y",
                "KydbzBtk6uc7M6dXwEgTEH2sphZxSPbmDSz6kUUHi4eUpSQuhEbq",
                "042596957532fc37e40486b910802ff45eeaa924548c0e1c080ef804e523ec3ed3"
                "ed0a9004acf927666eee18b7f5e8ad72ff100a3bb710a577256fd7ec81eb1cb3",
                "032596957532fc37e40486b910802ff45eeaa924548c0e1c080ef804e523ec3ed3",
                "1PM35qz2uwCDzcUJtiqDSudAaaLrWRw41L",
                "19ck9VKC6KjGxR9LJg4DNMRc45qFrJguvV")

        # in this case, the public_pair y value is less than 256**31, and so has a leading 00 byte.
        # This triggers a bug in the Python 2.7 version of to_bytes_32.
        do_test("ae2aaef5080b6e1704aab382a40a7c9957a40b4790f7df7faa04b14f4db56361",
                "5K8zSJ4zcV3UfkAKCFY5PomL6SRx2pYjaKfnAtMVh6zbhnAuPon",
                "L34GWeLdHcmw81W7JfAAPfQfH1F7u2s4v5QANdfTe1TEAYpjXoLL",
                "04f650fb572d1475950b63f5175c77e8b5ed9035a209d8fb5af5a04d6bc39b7323"
                "00186733fcfe3def4ace6feae8b82dd03cc31b7855307d33b0a039170f374962",
                "02f650fb572d1475950b63f5175c77e8b5ed9035a209d8fb5af5a04d6bc39b7323",
                "18fKPR8s1MQeckAsgya1sx6Z3WmFXd8wv8",
                "1DVJQzgnyCahXdoXdJ3tjGA3hrYVgKpvgK")


if __name__ == '__main__':
    unittest.main()



from pycoin.networks.bitcoinish import create_bitcoinish_network

network = create_bitcoinish_network(
    symbol="ARG", network_name="Argentum", subnet_name="mainnet",
    wif_prefix_hex="97", address_prefix_hex="17", pay_to_script_prefix_hex="05",
    bip32_prv_prefix_hex="0488ade4", bip32_pub_prefix_hex="0488b21e")



import unittest

from pycoin.ecdsa.secp256k1 import secp256k1_generator
from pycoin.ecdsa.intstream import to_bytes, from_bytes


class ECDSATestCase(unittest.TestCase):

    def test_infinity(self):
        infinity = secp256k1_generator.infinity()
        self.assertEqual(secp256k1_generator * 0, infinity)
        self.assertEqual(0 * secp256k1_generator, infinity)
        for _ in range(0, 100, 10):
            self.assertEqual(_ * infinity, infinity)
            self.assertEqual(infinity * _, infinity)
        g2 = secp256k1_generator * 2
        g2_neg = secp256k1_generator * -2
        self.assertEqual(g2 + g2_neg, infinity)
        self.assertEqual(g2 + infinity, g2)
        self.assertEqual(g2_neg + infinity, g2_neg)
        self.assertEqual(-g2, g2_neg)

    def test_multiply(self):
        g2 = secp256k1_generator * 2
        g2p = g2 * 1
        self.assertEqual(g2p, g2)
        g4 = g2 * 2
        self.assertEqual(g4, secp256k1_generator * 4)
        g8 = g2 * 4
        self.assertEqual(g8, secp256k1_generator * 8)
        g24 = g8 * 3
        self.assertEqual(g24, secp256k1_generator * 24)
        g_big = g2 * (71 ** 41)
        self.assertEqual(g_big, secp256k1_generator * ((2 * 71 ** 41) % secp256k1_generator.order()))

    def test_add(self):
        G = secp256k1_generator
        a, b = 2, 3
        self.assertEqual(a * G + b * G, (a + b) * G)
        a, b = 200, 300
        self.assertEqual(a * G + b * G, (a + b) * G)
        a, b = 71**41, 41**47
        self.assertEqual(a * G + b * G, (a + b) * G)

    def test_sign_simple(self):
        secret_exponent = 1
        public_pair = secp256k1_generator * secret_exponent
        self.assertEqual(public_pair, (
            55066263022277343669578718895168534326250603453777594175500187360389116729240,
            32670510020758816978083085130507043184471273380659243275938904335757337482424)
        )
        hash_value = 1
        sig = secp256k1_generator.sign(secret_exponent, hash_value)
        r = secp256k1_generator.verify(public_pair, hash_value, sig)
        self.assertTrue(r)
        r = secp256k1_generator.verify(public_pair, hash_value, (sig[0], sig[1] ^ 1))
        self.assertFalse(r)
        self.assertEqual(sig[0], 46340862580836590753275244201733144181782255593078084106116359912084275628184)
        self.assertIn(sig[1], [
            81369331955758484632176499244870227132558660296342819670803726373940306621624,
            34422757281557710791394485763817680720278903982732084711801436767577854872713
        ])

    def test_verify_simple(self):
        public_pair = secp256k1_generator * 1
        self.assertEqual(public_pair, (
            55066263022277343669578718895168534326250603453777594175500187360389116729240,
            32670510020758816978083085130507043184471273380659243275938904335757337482424)
        )
        hash_value = 1
        sig = (46340862580836590753275244201733144181782255593078084106116359912084275628184,
               81369331955758484632176499244870227132558660296342819670803726373940306621624)
        r = secp256k1_generator.verify(public_pair, hash_value, sig)
        self.assertEqual(r, True)

    def test_sign_verify(self):
        def do_test(secret_exponent, val_list):
            public_point = secret_exponent * secp256k1_generator
            for v in val_list:
                signature = secp256k1_generator.sign(secret_exponent, v)
                r = secp256k1_generator.verify(public_point, v, signature)
                assert r is True
                inv_sig = (signature[0], secp256k1_generator.order() - signature[1])
                r = secp256k1_generator.verify(public_point, v, inv_sig)
                assert r is True
                signature = signature[0], signature[1]+1
                r = secp256k1_generator.verify(public_point, v, signature)
                assert r is False

        val_list = [100, 20000, 30000000, 400000000000, 50000000000000000, 60000000000000000000000]

        do_test(0x1111111111111111111111111111111111111111111111111111111111111111, val_list)
        do_test(0xdddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddd, val_list)
        do_test(0x47f7616ea6f9b923076625b4488115de1ef1187f760e65f89eb6f4f7ff04b012, val_list)

    def test_custom_k(self):
        secret_exponent = 1
        sig_hash = 1

        def gen_k(*args):
            return 1

        signature = secp256k1_generator.sign(secret_exponent, sig_hash, gen_k)
        self.assertEqual(signature, (
            55066263022277343669578718895168534326250603453777594175500187360389116729240,
            55066263022277343669578718895168534326250603453777594175500187360389116729241
        ))

    def test_inverse_mod(self):
        prime = secp256k1_generator.curve().p()
        order = secp256k1_generator.order()
        for v in range(70):
            n = int(float("1e%d" % v))
            i = secp256k1_generator.inverse_mod(n, prime)
            assert n * i % prime == 1
            i = secp256k1_generator.inverse_mod(n, order)
            assert n * i % order == 1

    def test_endian(self):
        for e in ("big", "little"):
            assert from_bytes(to_bytes(768, 2, e), e) == 768
            assert from_bytes(to_bytes(3, 1, e), e) == 3
            assert from_bytes(to_bytes(66051, 3, e), e) == 66051


if __name__ == '__main__':
    unittest.main()



import unittest

from pycoin.cmds import ku
from pycoin.networks.registry import network_for_netcode

from .ToolTest import ToolTest


def make_tests_for_netcode(netcode):

    network = network_for_netcode(netcode)

    class KuTest(ToolTest):

        @classmethod
        def setUpClass(cls):
            cls.parser = ku.create_parser()
            cls.tool_name = "ku"

        def test_ku_create(self):
            output = self.launch_tool("ku create -w -n %s" % netcode).split("\n")
            bip32 = network.parse.bip32_prv(output[0])
            bip32_as_text = bip32.hwif(as_private=True)
            self.assertEqual(output[0], bip32_as_text)

    return KuTest


for netcode in ["BTC", "LTC", "BCH", "DOGE", "XTN"]:
    exec("%sTests = make_tests_for_netcode('%s')" % (netcode, netcode))


def main():
    unittest.main()


if __name__ == "__main__":
    main()



import unittest

from pycoin.encoding.b58 import a2b_base58, b2a_base58, a2b_hashed_base58, b2a_hashed_base58, is_hashed_base58_valid
from pycoin.encoding.base_conversion import from_long, to_long, EncodingError
from pycoin.encoding.bytes32 import to_bytes_32
from pycoin.encoding.hash import double_sha256, hash160
from pycoin.encoding.hexbytes import h2b
from pycoin.encoding.sec import is_sec_compressed, public_pair_to_hash160_sec, public_pair_to_sec, sec_to_public_pair
from pycoin.ecdsa.secp256k1 import secp256k1_generator
from pycoin.intbytes import iterbytes
from pycoin.symbols.btc import network


class EncodingTestCase(unittest.TestCase):

    def test_to_from_long(self):
        def do_test(as_int, prefix, as_rep, base):
            self.assertEqual((as_int, prefix), to_long(base, lambda v: v, iterbytes(as_rep)))
            self.assertEqual(as_rep, from_long(as_int, prefix, base, lambda v: v))

        do_test(10000101, 2, h2b("00009896e5"), 256)
        do_test(10000101, 3, h2b("0000009896e5"), 256)
        do_test(1460765565493402645157733592332121663123460211377, 1,
                h2b("00ffdefe4f4875cf119fc3d8f4a09ae37ec4cc42b1"), 256)

    def test_to_bytes_32(self):
        for i in range(256):
            v = to_bytes_32(i)
            self.assertEqual(v, b'\0' * 31 + bytes(bytearray([i])))
        for i in range(256, 512):
            v = to_bytes_32(i)
            self.assertEqual(v, b'\0' * 30 + bytes(bytearray([1, i & 0xff])))

    def test_to_from_base58(self):
        def do_test(as_text, as_bin):
            self.assertEqual(as_bin, a2b_base58(as_text))
            self.assertEqual(as_text, b2a_base58(as_bin))

        do_test("1abcdefghijkmnpqrst", h2b("0001935c7cf22ab9be1962aee48c7b"))
        do_test("1CASrvcpMMTa4dz4DmYtAqcegCtdkhjvdn",
                h2b("007a72b6fa63de36c4abc60a68b52d7f33e3d7cd3ec4babd39"))
        do_test("1111111111111111aaaa11aa",
                h2b("00000000000000000000000000000000436e7a51290b"))
        do_test("123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz",
                h2b("000111d38e5fc9071ffcd20b4a763cc9ae4f252bb4e48fd66a835e252ada93ff480d6d"
                    "d43dc62a641155a5"))

    def test_to_from_hashed_base58(self):
        def do_test(as_text, as_bin):
            self.assertEqual(as_text, b2a_hashed_base58(as_bin))
            self.assertEqual(as_bin, a2b_hashed_base58(as_text))
            self.assertTrue(is_hashed_base58_valid(as_text))
            bogus_text = as_text[:-1] + chr(1+ord(as_text[-1]))
            self.assertFalse(is_hashed_base58_valid(bogus_text))

        do_test("14nr3dMd4VwNpFhFECU1A6imi", h2b("0001935c7cf22ab9be1962aee48c7b"))
        do_test("1CASrvcpMMTa4dz4DmYtAqcegCtdkhjvdn", h2b("007a72b6fa63de36c4abc60a68b52d7f33e3d7cd3e"))
        do_test("11111111111111114njGbaozZJui9o",
                h2b("00000000000000000000000000000000436e7a51290b"))
        do_test("1mLRia5CbfDB9752zxvtrpnkigecaYWUSQNLJGECA8641ywusqomjhfdb6EM7bXGj1Gb",
                h2b("000111d38e5fc9071ffcd20b4a763cc9ae4f252bb4e48fd66a835e252ada93ff480d6dd43dc62a641155a561616161"))

    def test_double_sha256(self):
        def do_test(blob, expected_hash):
            self.assertEqual(double_sha256(blob), expected_hash)

        do_test(b"This is a test",
                h2b("eac649d431aa3fc2d5749d1a5021bba7812ec83b8a59fa840bff75c17f8a665c"))
        do_test(b"The quick brown fox jumps over the lazy dogs",
                h2b("8a356588797a901a11031779d4787ad0457eb082c56bd9b657157acf31bae6c4"))
        do_test(b'\x74' * 10000,
                h2b("6e4d7736aa373c4718eef2b94528fed57519a0bdc3a8f4300aee372cbedea9a0"))

    def test_hash160(self):
        def do_test(blob, expected_hash):
            self.assertEqual(hash160(blob), expected_hash)

        do_test(b"This is a test", h2b("18ac98fa2a2412ddb75de60459b52acd98f2d972"))
        do_test(b"The quick brown fox jumps over the lazy dogs",
                h2b("76c9d1f3aa5226554e20475f919aadd174f7e9b7"))
        do_test(b'\x74' * 10000, h2b("a961070296677401a57eae0d96d14d5a880a2c41"))

    def test_wif_to_from_secret_exponent(self):
        def do_test(as_secret_exponent, as_wif, is_compressed):
            key = network.keys.private(as_secret_exponent, is_compressed=is_compressed)
            self.assertEqual(as_wif, key.wif())
            key = network.parse.wif(as_wif)
            se = key.secret_exponent()
            comp = key.is_compressed()
            self.assertEqual(se, as_secret_exponent)
            self.assertEqual(comp, is_compressed)

        WIF_LIST = [
            "5HwoXVkHoRM8sL2KmNRS217n1g8mPPBomrY7yehCuXC1115WWsh",
            "5J5KUK3VXP8HUefNVYPxwxVRokScZdWXpu1Tj8LfaAXMqHzMmbk",
            "5JCqR8LhFLuS5yJRDiNVsus5bpkTjsqFswUoUbz8EorifYA4TwJ",
            "5JLMMwdtyJgahHwTwtM2osEjPu4Jv89yvyx9E5dauTC5Vs6EjBA",
            "5JTsJkw6hGTjJcaWg4KZjpcPByNA6NUhz2RUyZH3a6XSL7vAYmy",
            "5JbPFaEJREEsuwDZQEJ6fmz2z3g1GcoS34tpj2vWEjroARtCMBF",
            "5JiuCPXW9C22XFrc8QGdbjMgn7yrSs8A67NAUWZxuPC9ziUizQP",
            "5JrR9Cphs9oB8aVeraFAXgjLaCHhd7St99qWDzDRa2XWq3RVw7d",
            "5Jyw627ub7aKju8hakDhTe6zNGbYoMmcCCJqyTrtEfrsfLDreVt",
            "5K7T2qR7K5MUMDmkJvCEPbUeALuPyc6LFEnBiwWLuKCEVdBp8qV",
            "5KExyeiK338cxYQo36AmKYrHxRDF9rR4JHFXUR9oZxXbKue7gdL",
            "5KNUvU1WkzumZs3qmG9JFWDwkVX6L6jnMKisDtoGEbrxACzxk6T",
            "5KVzsHJiUxgvBBgtVS7qBTbbYZpwWM4WQNCCyNSiuFCJzYMxg8H",
            "5KdWp6bvCvU4nWKwDc6N7QyFLe8ngbPETQfYir6BZtXfpsnSrGS",
        ]
        SE_LIST = [int(c * 64, 16) for c in "123456789abcde"]
        for se, wif in zip(SE_LIST, WIF_LIST):
            do_test(se, wif, is_compressed=False)

    def test_public_pair_to_sec(self):
        def do_test(as_public_pair, as_sec, is_compressed, as_hash160_sec, as_bitcoin_address):
            self.assertEqual(sec_to_public_pair(as_sec, secp256k1_generator), as_public_pair)
            self.assertEqual(public_pair_to_sec(as_public_pair, compressed=is_compressed), as_sec)
            self.assertEqual(is_sec_compressed(as_sec), is_compressed)
            self.assertEqual(public_pair_to_hash160_sec(as_public_pair, compressed=is_compressed),
                             as_hash160_sec)

            self.assertEqual(network.address.for_p2pkh(as_hash160_sec), as_bitcoin_address)
            self.assertIsNotNone(network.parse.address(as_bitcoin_address))
            bad_address = as_bitcoin_address[:17] + chr(ord(as_bitcoin_address[17]) + 1) + as_bitcoin_address[18:]
            self.assertIsNone(network.parse.address(bad_address))

        SEC_TEST_DATA = [
            (
                (35826991941973211494003564265461426073026284918572421206325859877044495085994,
                 25491041833361137486709012056693088297620945779048998614056404517283089805761),
                "034f355bdcb7cc0af728ef3cceb9615d90684bb5b2ca5f859ab0f0b704075871aa",
                True,
                "fc7250a211deddc70ee5a2738de5f07817351cef",
                "1Q1pE5vPGEEMqRcVRMbtBK842Y6Pzo6nK9"
            ),
            (
                (31855367722742370537280679280108010854876607759940877706949385967087672770343,
                 46659058944867745027460438812818578793297503278458148978085384795486842595210),
                "02466d7fcae563e5cb09a0d1870bb580344804617879a14949cf22285f1bae3f27",
                True,
                "531260aa2a199e228c537dfa42c82bea2c7c1f4d",
                "18aF6pYXKDSXjXHpidt2G6okdVdBr8zA7z"
            ),
            (
                (27341391395138457474971175971081207666803680341783085051101294443585438462385,
                 26772005640425216814694594224987412261034377630410179754457174380653265224672),
                "023c72addb4fdf09af94f0c94d7fe92a386a7e70cf8a1d85916386bb2535c7b1b1",
                True,
                "3bc28d6d92d9073fb5e3adf481795eaf446bceed",
                "16Syw4SugWs4siKbK8cuxJXM2ukh2GKpRi"
            ),
            (
                (35826991941973211494003564265461426073026284918572421206325859877044495085994,
                 25491041833361137486709012056693088297620945779048998614056404517283089805761),
                "044f355bdcb7cc0af728ef3cceb9615d90684bb5b2ca5f859ab0f0b704075871aa"
                "385b6b1b8ead809ca67454d9683fcf2ba03456d6fe2c4abe2b07f0fbdbb2f1c1",
                False,
                "e4e517ee07984a4000cd7b00cbcb545911c541c4",
                "1MsHWS1BnwMc3tLE8G35UXsS58fKipzB7a"
            ),
            (
                (31855367722742370537280679280108010854876607759940877706949385967087672770343,
                 46659058944867745027460438812818578793297503278458148978085384795486842595210),
                "04466d7fcae563e5cb09a0d1870bb580344804617879a14949cf22285f1bae3f27"
                "6728176c3c6431f8eeda4538dc37c865e2784f3a9e77d044f33e407797e1278a",
                False,
                "b256082b934fe782adbacaafeadfca64c52a5384",
                "1HFxLkPTtMZeo5mDpZn6CF9sh4h2ycknwr"
            ),
            (
                (27341391395138457474971175971081207666803680341783085051101294443585438462385,
                 26772005640425216814694594224987412261034377630410179754457174380653265224672),
                "043c72addb4fdf09af94f0c94d7fe92a386a7e70cf8a1d85916386bb2535c7b1b1"
                "3b306b0fe085665d8fc1b28ae1676cd3ad6e08eaeda225fe38d0da4de55703e0",
                False,
                "edf6bbd7ba7aad222c2b28e6d8d5001178e3680c",
                "1NhEipumt9Pug6pwTqMNRXhBG84K39Ebbi"
            ),
        ]

        for public_pair, sec, compressed, hash160_sec, bitcoin_address in SEC_TEST_DATA:
            do_test(public_pair, h2b(sec), compressed, h2b(hash160_sec), bitcoin_address)

    def test_sec(self):
        pair_blob = h2b("0679be667ef9dcbbac55a06295ce870b07029bfcdb2dce28d959f2815b16f81798483a"
                        "da7726a3c4655da4fbfc0e1108a8fd17b448a68554199c47d08ffb10d4b8")
        sec_to_public_pair(pair_blob, secp256k1_generator, strict=False)
        try:
            sec_to_public_pair(pair_blob, secp256k1_generator, strict=True)
            self.fail("sec_to_public_pair unexpectedly succeeded")
        except EncodingError:
            pass


if __name__ == '__main__':
    unittest.main()



from .Generator import Generator
from .native.openssl import create_OpenSSLOptimizations, NID_secp256k1
from .native.secp256k1 import LibSECP256K1Optimizations

# from http://www.secg.org/sec2-v2.pdf

_a = 0x0000000000000000000000000000000000000000000000000000000000000000
_b = 0x0000000000000000000000000000000000000000000000000000000000000007
_p = 0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffefffffc2f
_Gx = 0x79be667ef9dcbbac55a06295ce870b07029bfcdb2dce28d959f2815b16f81798
_Gy = 0x483ada7726a3c4655da4fbfc0e1108a8fd17b448a68554199c47d08ffb10d4b8
_r = 0xfffffffffffffffffffffffffffffffebaaedce6af48a03bbfd25e8cd0364141


# include optimizations from libsecp256k1 and openssl, if available

class GeneratorWithOptimizations(LibSECP256K1Optimizations, create_OpenSSLOptimizations(NID_secp256k1), Generator):
    pass


secp256k1_generator = GeneratorWithOptimizations(_p, _a, _b, (_Gx, _Gy), _r)



from pycoin.coins.bitcoin.Solver import BitcoinSolver
from .SolutionChecker import GroestlcoinSolutionChecker


class GroestlcoinSolver(BitcoinSolver):
    SolutionChecker = GroestlcoinSolutionChecker



import hashlib

from pycoin.encoding.hexbytes import bytes_as_revhex


def sha256(data):
    return bytes_as_revhex(hashlib.sha256(data).digest())


def groestlHash(data):
    """Groestl-512 compound hash."""
    try:
        import groestlcoin_hash
    except ImportError:
        t = 'Groestlcoin requires the groestlcoin_hash package ("pip install groestlcoin_hash").'
        print(t)
        raise ImportError(t)

    return bytes_as_revhex(groestlcoin_hash.getHash(data, len(data)))



import io

from hashlib import sha256

from ...encoding.hash import double_sha256
from ...encoding.bytes32 import from_bytes_32
from ...intbytes import byte2int, indexbytes

from ..SolutionChecker import SolutionChecker, ScriptError
from pycoin.satoshi import errno

from pycoin.satoshi.satoshi_struct import stream_struct
from pycoin.satoshi.satoshi_string import stream_satoshi_string

from pycoin.satoshi.flags import (
    SIGHASH_NONE, SIGHASH_SINGLE, SIGHASH_ANYONECANPAY,
    VERIFY_DISCOURAGE_UPGRADABLE_WITNESS_PROGRAM, VERIFY_CLEANSTACK, VERIFY_WITNESS
)

from .ScriptTools import BitcoinScriptTools


ZERO32 = b'\0' * 32


class SegwitChecker(SolutionChecker):
    # you must set VM
    # you must set ScriptTools

    V0_len20_prefix = BitcoinScriptTools.compile("OP_DUP OP_HASH160")
    V0_len20_postfix = BitcoinScriptTools.compile("OP_EQUALVERIFY OP_CHECKSIG")

    OP_0 = BitcoinScriptTools.int_for_opcode("OP_0")
    OP_1 = BitcoinScriptTools.int_for_opcode("OP_1")
    OP_16 = BitcoinScriptTools.int_for_opcode("OP_16")

    def _make_witness_sighash_f(self, tx_in_idx):

        def witness_signature_for_hash_type(hash_type, sig_blobs, vm):
            return self._signature_for_hash_type_segwit(
                vm.script[vm.begin_code_hash:], tx_in_idx, hash_type)

        return witness_signature_for_hash_type

    def _puzzle_script_for_len20_segwit(self, witness_program):
        return self.V0_len20_prefix + self.ScriptTools.compile_push_data_list(
            [witness_program]) + self.V0_len20_postfix

    def _check_witness_program_v0(self, witness_solution_stack, witness_program):
        size = len(witness_program)
        if size == 32:
            if len(witness_solution_stack) == 0:
                raise ScriptError("witness program witness empty", errno.WITNESS_PROGRAM_WITNESS_EMPTY)
            puzzle_script = witness_solution_stack[-1]
            if sha256(puzzle_script).digest() != witness_program:
                raise ScriptError("witness program mismatch", errno.WITNESS_PROGRAM_MISMATCH)
            stack = list(witness_solution_stack[:-1])
        elif size == 20:
            # special case for pay-to-pubkeyhash; signature + pubkey in witness
            if len(witness_solution_stack) != 2:
                raise ScriptError("witness program mismatch", errno.WITNESS_PROGRAM_MISMATCH)
            puzzle_script = self._puzzle_script_for_len20_segwit(witness_program)
            stack = list(witness_solution_stack)
        else:
            raise ScriptError("witness program wrong length", errno.WITNESS_PROGRAM_WRONG_LENGTH)
        return stack, puzzle_script

    def _witness_program_version(self, script):
        size = len(script)
        if size < 4 or size > 42:
            return None
        first_opcode = byte2int(script)
        if indexbytes(script, 1) + 2 != size:
            return None
        if first_opcode == self.OP_0:
            return 0
        if self.OP_1 <= first_opcode <= self.OP_16:
            return first_opcode - self.OP_1 + 1
        return None

    def _hash_prevouts(self, hash_type):
        if hash_type & SIGHASH_ANYONECANPAY:
            return ZERO32
        f = io.BytesIO()
        for tx_in in self.tx.txs_in:
            f.write(tx_in.previous_hash)
            stream_struct("L", f, tx_in.previous_index)
        return double_sha256(f.getvalue())

    def _hash_sequence(self, hash_type):
        if (
                (hash_type & SIGHASH_ANYONECANPAY) or
                ((hash_type & 0x1f) == SIGHASH_SINGLE) or
                ((hash_type & 0x1f) == SIGHASH_NONE)
        ):
            return ZERO32

        f = io.BytesIO()
        for tx_in in self.tx.txs_in:
            stream_struct("L", f, tx_in.sequence)
        return double_sha256(f.getvalue())

    def _hash_outputs(self, hash_type, tx_in_idx):
        txs_out = self.tx.txs_out
        if hash_type & 0x1f == SIGHASH_SINGLE:
            if tx_in_idx >= len(txs_out):
                return ZERO32
            txs_out = txs_out[tx_in_idx:tx_in_idx+1]
        elif hash_type & 0x1f == SIGHASH_NONE:
            return ZERO32
        f = io.BytesIO()
        for tx_out in txs_out:
            stream_struct("QS", f, tx_out.coin_value, tx_out.script)
        return double_sha256(f.getvalue())

    def _segwit_signature_preimage(self, script, tx_in_idx, hash_type):
        f = io.BytesIO()
        stream_struct("L", f, self.tx.version)
        # calculate hash prevouts
        f.write(self._hash_prevouts(hash_type))
        f.write(self._hash_sequence(hash_type))
        tx_in = self.tx.txs_in[tx_in_idx]
        f.write(tx_in.previous_hash)
        stream_struct("L", f, tx_in.previous_index)
        tx_out = self.tx.unspents[tx_in_idx]
        stream_satoshi_string(f, script)
        stream_struct("Q", f, tx_out.coin_value)
        stream_struct("L", f, tx_in.sequence)
        f.write(self._hash_outputs(hash_type, tx_in_idx))
        stream_struct("L", f, self.tx.lock_time)
        stream_struct("L", f, hash_type)
        return f.getvalue()

    def _signature_for_hash_type_segwit(self, script, tx_in_idx, hash_type):
        return from_bytes_32(double_sha256(self._segwit_signature_preimage(script, tx_in_idx, hash_type)))

    def witness_program_tuple(self, tx_context, puzzle_script, solution_stack, flags, is_p2sh):
        if not flags & VERIFY_WITNESS:
            return

        witness_version = self._witness_program_version(puzzle_script)
        if witness_version is None:
            if len(tx_context.witness_solution_stack) > 0:
                raise ScriptError("witness unexpected", errno.WITNESS_UNEXPECTED)
        else:
            witness_program = puzzle_script[2:]
            if len(solution_stack) > 0:
                err = errno.WITNESS_MALLEATED_P2SH if is_p2sh else errno.WITNESS_MALLEATED
                raise ScriptError("script sig is not blank on segwit input", err)

            for s in tx_context.witness_solution_stack:
                if len(s) > self.VM.MAX_BLOB_LENGTH:
                    raise ScriptError("pushing too much data onto stack", errno.PUSH_SIZE)

            if witness_version == 0:
                stack, puzzle_script = self._check_witness_program_v0(
                    tx_context.witness_solution_stack, witness_program)
                sighash_f = self._make_witness_sighash_f(tx_context.tx_in_idx)
                return puzzle_script, stack, flags | VERIFY_CLEANSTACK, sighash_f
            elif flags & VERIFY_DISCOURAGE_UPGRADABLE_WITNESS_PROGRAM:
                raise ScriptError(
                    "this version witness program not yet supported", errno.DISCOURAGE_UPGRADABLE_WITNESS_PROGRAM)



import unittest

from pycoin.ecdsa.secp256k1 import secp256k1_generator
from pycoin.encoding.hexbytes import b2h, b2h_rev
from pycoin.intbytes import int2byte
from pycoin.networks.registry import network_for_netcode
from pycoin.satoshi.der import sigdecode_der, sigencode_der


PRIV_KEYS = (
    2330949616242593315303241053456316633827293588958882755297900732239663851861,
    4437411780076344925846479906614060621668407514498402815534040340772719979673,
    14311886404724799688521454580288220586308410691395501373612453626821267193196,
    16404731722033649474165521611800542240555275746052963990137782680023514762282,
    92715304942310420502826004911529506622922082818576946681102234225452853924813,
    103235678552410630318322729483874198805317322052500844759252733409163632402845,
)


def sigcheck(a_key, a_hash_for_sig, a_sig):
    """
    Returns True if a_key was used to generate a_sig from a_hash_for_sig;
    False otherwise.
    """
    r, s = sigdecode_der(a_sig)

    return secp256k1_generator.verify(a_key.public_pair(), a_hash_for_sig, (r, s))


def sigmake(a_key, a_hash_for_sig, a_sig_type):
    """
    Signs a_hash_for_sig with a_key and returns a DER-encoded signature
    with a_sig_type appended.
    """
    order = secp256k1_generator.order()
    r, s = secp256k1_generator.sign(a_key.secret_exponent(), a_hash_for_sig)

    if s + s > order:
        s = order - s

    return sigencode_der(r, s) + int2byte(a_sig_type)


class SighashSingleTest(unittest.TestCase):

    def test_sighash_single(self):
        for netcode in ["BTC", "XTN"]:
            self._test_sighash_single(network_for_netcode(netcode))

    def _test_sighash_single(self, network):
        flags = network.validator.flags

        k0, k1, k2, k3, k4, k5 = [
            network.keys.private(secret_exponent=se, is_compressed=True) for se in PRIV_KEYS]

        # Fake a coinbase transaction
        coinbase_tx = network.tx.coinbase_tx(k0.sec(), 500000000)
        for k in [k1, k2]:
            coinbase_tx.txs_out.append(network.tx.TxOut(
                1000000000, network.script.compile('%s OP_CHECKSIG' % b2h(k.sec()))))

        self.assertEqual('2acbe1006f7168bad538b477f7844e53de3a31ffddfcfc4c6625276dd714155a',
                         b2h_rev(coinbase_tx.hash()))

        # Make the test transaction
        txs_in = [
            network.tx.TxIn(coinbase_tx.hash(), 0),
            network.tx.TxIn(coinbase_tx.hash(), 1),
            network.tx.TxIn(coinbase_tx.hash(), 2),
        ]
        txs_out = [
            network.tx.TxOut(900000000, network.contract.for_address(k3.address())),
            network.tx.TxOut(800000000, network.contract.for_address(k4.address())),
            network.tx.TxOut(800000000, network.contract.for_address(k5.address())),
        ]
        tx = network.tx(1, txs_in, txs_out)
        tx.set_unspents(coinbase_tx.txs_out)

        self.assertEqual('791b98ef0a3ac87584fe273bc65abd89821569fd7c83538ac0625a8ca85ba587', b2h_rev(tx.hash()))

        sig_type = flags.SIGHASH_SINGLE

        solution_checker = network.tx.SolutionChecker(tx)
        sig_hash = solution_checker._signature_hash(coinbase_tx.txs_out[0].script, 0, sig_type)
        self.assertEqual(0xcc52d785a3b4133504d1af9e60cd71ca422609cb41df3a08bbb466b2a98a885e, sig_hash)

        sig = sigmake(k0, sig_hash, sig_type)
        self.assertTrue(sigcheck(k0, sig_hash, sig[:-1]))

        tx.txs_in[0].script = network.script.compile(b2h(sig))
        self.assertTrue(tx.is_solution_ok(0))

        sig_hash = solution_checker._signature_hash(coinbase_tx.txs_out[1].script, 1, sig_type)
        self.assertEqual(0x93bb883d70fccfba9b8aa2028567aca8357937c65af7f6f5ccc6993fd7735fb7, sig_hash)

        sig = sigmake(k1, sig_hash, sig_type)
        self.assertTrue(sigcheck(k1, sig_hash, sig[:-1]))

        tx.txs_in[1].script = network.script.compile(b2h(sig))
        self.assertTrue(tx.is_solution_ok(1))

        sig_hash = solution_checker._signature_hash(coinbase_tx.txs_out[2].script, 2, sig_type)
        self.assertEqual(0x53ef7f67c3541bffcf4e0d06c003c6014e2aa1fb38ff33240b3e1c1f3f8e2a35, sig_hash)

        sig = sigmake(k2, sig_hash, sig_type)
        self.assertTrue(sigcheck(k2, sig_hash, sig[:-1]))

        tx.txs_in[2].script = network.script.compile(b2h(sig))
        self.assertTrue(tx.is_solution_ok(2))

        sig_type = flags.SIGHASH_SINGLE | flags.SIGHASH_ANYONECANPAY

        sig_hash = solution_checker._signature_hash(coinbase_tx.txs_out[0].script, 0, sig_type)
        self.assertEqual(0x2003393d246a7f136692ce7ab819c6eadc54ffea38eb4377ac75d7d461144e75, sig_hash)

        sig = sigmake(k0, sig_hash, sig_type)
        self.assertTrue(sigcheck(k0, sig_hash, sig[:-1]))

        tx.txs_in[0].script = network.script.compile(b2h(sig))
        self.assertTrue(tx.is_solution_ok(0))

        sig_hash = solution_checker._signature_hash(coinbase_tx.txs_out[1].script, 1, sig_type)
        self.assertEqual(0xe3f469ac88e9f35e8eff0bd8ad4ad3bf899c80eb7645947d60860de4a08a35df, sig_hash)

        sig = sigmake(k1, sig_hash, sig_type)
        self.assertTrue(sigcheck(k1, sig_hash, sig[:-1]))

        tx.txs_in[1].script = network.script.compile(b2h(sig))
        self.assertTrue(tx.is_solution_ok(1))

        sig_hash = solution_checker._signature_hash(coinbase_tx.txs_out[2].script, 2, sig_type)
        self.assertEqual(0xbacd7c3ab79cad71807312677c1788ad9565bf3c00ab9a153d206494fb8b7e6a, sig_hash)

        sig = sigmake(k2, sig_hash, sig_type)
        self.assertTrue(sigcheck(k2, sig_hash, sig[:-1]))

        tx.txs_in[2].script = network.script.compile(b2h(sig))
        self.assertTrue(tx.is_solution_ok(2))


if __name__ == "__main__":
    unittest.main()



import unittest

from pycoin.ecdsa.Point import Point
from pycoin.ecdsa.Generator import Generator
from pycoin.ecdsa.Curve import Curve


class GeneratorTestCase(unittest.TestCase):

    c23 = Curve(23, 1, 1)

    def do_test_add(self, c, x1, y1, x2, y2, x3, y3):
        """We expect that on curve c, (x1,y1) + (x2, y2) = (x3, y3)."""
        p1 = Point(x1, y1, c)
        p2 = Point(x2, y2, c)
        p3 = p1 + p2
        print("%s + %s = %s" % (p1, p2, p3))
        if p3[0] != x3 or p3[1] != y3:
            raise unittest.FailedTest("Failure: should give (%d,%d)." % (x3, y3))
        else:
            print(" Good.")

    def do_test_double(self, c, x1, y1, x3, y3):
        """We expect that on curve c, 2*(x1,y1) = (x3, y3)."""
        p1 = Point(x1, y1, c)
        p3 = p1 + p1
        print("%s doubled = %s" % (p1, p3))
        if p3[0] != x3 or p3[1] != y3:
            raise unittest.FailedTest("Failure: should give (%d,%d)." % (x3, y3))
        else:
            print(" Good.")

    def do_test_double_infinity(self, c):
        """We expect that on curve c, 2*INFINITY = INFINITY."""
        p1 = c._infinity
        p3 = p1 + p1
        print("%s doubled = %s" % (p1, p3))
        if p3[0] != p1[0] or p3[1] != p1[1]:
            raise unittest.FailedTest("Failure: should give (%d,%d)." % (p1[0], p1[1]))
        else:
            print(" Good.")

    def do_test_multiply(self, c, x1, y1, m, x3, y3):
        """We expect that on curve c, m*(x1,y1) = (x3,y3)."""
        p1 = Point(x1, y1, c)
        p3 = p1 * m
        print("%s * %d = %s" % (p1, m, p3))
        if p3[0] != x3 or p3[1] != y3:
            raise unittest.FailedTest("Failure: should give (%d,%d)." % (x3, y3))
        else:
            print(" Good.")

    def test_all(self):
        # A few tests from X9.62 B.3:

        self.do_test_add(self.c23, 3, 10, 9, 7, 17, 20)
        self.do_test_double(self.c23, 3, 10, 7, 12)
        self.do_test_add(self.c23, 3, 10, 3, 10, 7, 12)  # (Should just invoke double.)
        self.do_test_multiply(self.c23, 3, 10, 2, 7, 12)

        self.do_test_double_infinity(self.c23)

        # From X9.62 I.1 (p. 96):

        g = Point(13, 7, self.c23)

        check = self.c23._infinity
        for i in range(8):
            p = (i % 7) * g
            print("%s * %d = %s, expected %s . . ." % (g, i, p, check))
            self.assertEqual(p, check)
            check = check + g

        # NIST Curve P-192:
        p = 6277101735386680763835789423207666416083908700390324961279
        r = 6277101735386680763835789423176059013767194773182842284081
        #  s = 0x3045ae6fc8422f64ed579528d38120eae12196d5L
        #  c = 0x3099d2bbbfcb2538542dcd5fb078b6ef5f3d6fe2c745de65
        b = 0x64210519e59c80e70fa7e9ab72243049feb8deecc146b9b1
        Gx = 0x188da80eb03090f67cbf20eb43a18800f4ff0afd82ff1012
        Gy = 0x07192b95ffc8da78631011ed6b24cdd573f977a11e794811

        p192 = Generator(p, -3, b, (Gx, Gy), r)

        # Checking against some sample computations presented
        # in X9.62:

        d = 651056770906015076056810763456358567190100156695615665659
        Q = d * p192
        if Q[0] != 0x62B12D60690CDCF330BABAB6E69763B471F994DD702D16A5:
            raise unittest.FailedTest("p192 * d came out wrong.")
        else:
            print("p192 * d came out right.")

        k = 6140507067065001063065065565667405560006161556565665656654
        R = k * p192
        if R[0] != 0x885052380FF147B734C330C43D39B2C4A89F29B0F749FEAD \
           or R[1] != 0x9CF9FA1CBEFEFB917747A3BB29C072B9289C2547884FD835:
            raise unittest.FailedTest("k * p192 came out wrong.")
        else:
            print("k * p192 came out right.")

        u1 = 2563697409189434185194736134579731015366492496392189760599
        u2 = 6266643813348617967186477710235785849136406323338782220568
        temp = u1 * p192 + u2 * Q
        if temp[0] != 0x885052380FF147B734C330C43D39B2C4A89F29B0F749FEAD \
           or temp[1] != 0x9CF9FA1CBEFEFB917747A3BB29C072B9289C2547884FD835:
            raise unittest.FailedTest("u1 * p192 + u2 * Q came out wrong.")
        else:
            print("u1 * p192 + u2 * Q came out right.")



import unittest

from pycoin.encoding.b58 import a2b_hashed_base58, b2a_hashed_base58
from pycoin.networks.registry import network_for_netcode
from pycoin.networks.registry import network_codes
from pycoin.symbols.btc import network as BTC
from pycoin.symbols.xtn import network as XTN

NETCODES = "BTC XTN DOGE".split()


def change_prefix(address, new_prefix):
    return b2a_hashed_base58(new_prefix + a2b_hashed_base58(address)[1:])


P2PKH_ADDRESSES = [
    "1BgGZ9tcN4rm9KBzDn7KprQz87SZ26SAMH", "1EHNa6Q4Jz2uvNExL497mE43ikXhwF6kZm",
    "1cMh228HTCiwS8ZsaakH8A8wze1JR5ZsP", "1LagHJk2FyCV2VzrNHVqg3gYG4TSYwDV4m",
    "1CUNEBjYrCn2y1SdiUMohaKUi4wpP326Lb", "1NZUP3JAc9JkmbvmoTv7nVgZGtyJjirKV1"]

P2SH_ADDRESSES = [
    '3CNHUhP3uyB9EUtRLsmvFUmvGdjGdkTxJw', '3EyPVdtVrtMJ1XwPT9oiBrQysGpRY8LE9K',
    '32JNcZWZqMX72bpzzgFLhkX56WviowgUtS', '3MGhCrETosWs7fhHVPAS6g3UQakA7Xz3wb',
    '3DAP9jDzQ76R4B94qa2Q8CgQrbEXvUoghh', '3PFVJancA3d8rmdCvZaiD83VRRG2Em15Ge']


class KeyUtilsTest(unittest.TestCase):

    def test_address_valid_btc(self):
        for address in P2PKH_ADDRESSES:
            self.assertEqual(BTC.parse.p2pkh(address).address(), address)
            a = address[:-1] + chr(ord(address[-1])+1)
            self.assertIsNone(BTC.parse.address(a))

        for address in P2PKH_ADDRESSES:
            self.assertIsNone(BTC.parse.p2sh(address))
            self.assertEqual(BTC.parse.p2pkh(address).address(), address)

        for address in P2SH_ADDRESSES:
            self.assertEqual(address[0], "3")
            self.assertEqual(BTC.parse.p2sh(address).address(), address)
            self.assertIsNone(BTC.parse.p2pkh(address))

    def test_is_wif_valid(self):
        WIFS = ["KwDiBf89QgGbjEhKnhXJuH7LrciVrZi3qYjgd9M7rFU73sVHnoWn",
                "5HpHagT65TZzG1PH3CSu63k8DbpvD8s5ip4nEB3kEsreAnchuDf",
                "KwDiBf89QgGbjEhKnhXJuH7LrciVrZi3qYjgd9M7rFU74NMTptX4",
                "5HpHagT65TZzG1PH3CSu63k8DbpvD8s5ip4nEB3kEsreAvUcVfH"]

        for wif in WIFS:
            self.assertEqual(BTC.parse.wif(wif).wif(), wif)
            a = wif[:-1] + chr(ord(wif[-1])+1)
            self.assertIsNone(BTC.parse.wif(a))

        NETWORK_NAMES = network_codes()
        for netcode in NETWORK_NAMES:
            network = network_for_netcode(netcode)
            if not getattr(network, "Key", None):
                continue
            for se in range(1, 10):
                key = network.Key(secret_exponent=se)
                for tv in [True, False]:
                    wif = key.wif(is_compressed=tv)
                    self.assertEqual(network.parse.wif(wif).wif(), wif)
                    a = wif[:-1] + chr(ord(wif[-1])+1)
                    self.assertIsNone(network.parse.wif(a))

    def test_is_public_private_bip32_valid(self):
        from pycoin.networks.registry import network_for_netcode
        WALLET_KEYS = ["foo", "1", "2", "3", "4", "5"]

        # not all networks support BIP32 yet
        for netcode in NETCODES:
            network = network_for_netcode(netcode)
            for wk in WALLET_KEYS:
                wallet = network.keys.bip32_seed(wk.encode("utf8"))
                text = wallet.hwif(as_private=True)
                self.assertEqual(network.parse.bip32_prv(text).hwif(as_private=True), text)
                self.assertIsNone(network.parse.bip32_pub(text))
                a = text[:-1] + chr(ord(text[-1])+1)
                self.assertIsNone(network.parse.bip32_prv(a))
                self.assertIsNone(network.parse.bip32_pub(a))
                text = wallet.hwif(as_private=False)
                self.assertIsNone(network.parse.bip32_prv(text))
                self.assertEqual(network.parse.bip32_pub(text).hwif(), text)
                a = text[:-1] + chr(ord(text[-1])+1)
                self.assertIsNone(network.parse.bip32_prv(a))
                self.assertIsNone(network.parse.bip32_pub(a))

    def test_key_limits(self):
        nc = 'BTC'
        cc = b'000102030405060708090a0b0c0d0e0f'
        order = BTC.keys.private(1)._generator.order()

        # BRAIN DAMAGE: hack
        BIP32Node = BTC.keys.bip32_seed(b"foo").__class__
        for k in -1, 0, order, order + 1:
            self.assertRaises(BTC.keys.InvalidSecretExponentError, BTC.keys.private, secret_exponent=k)
            self.assertRaises(BTC.keys.InvalidSecretExponentError, BIP32Node, nc, cc, secret_exponent=k)

        for i in range(1, 512):
            BTC.keys.private(secret_exponent=i)
            BIP32Node(cc, secret_exponent=i)

    def test_repr(self):
        key = XTN.keys.private(secret_exponent=273)

        address = key.address()
        pub_k = XTN.parse(address)
        self.assertEqual(repr(pub_k),  '<mhDVBkZBWLtJkpbszdjZRkH1o5RZxMwxca>')

        wif = key.wif()
        priv_k = XTN.parse.wif(wif)
        self.assertEqual(
            repr(priv_k),
            'private_for <XTNSEC:0264e1b1969f9102977691a40431b0b672055dcf31163897d996434420e6c95dc9>')


if __name__ == '__main__':
    unittest.main()



import io
import pytest
import unittest

from pycoin.encoding.hexbytes import h2b
from pycoin.symbols.grs import network as GroestlcoinMainnet


pytest.importorskip('groestlcoin_hash')

# Block 2000000
BLOCK_HASH = '00000000000434d5b8d1c3308df7b6e3fd773657dfb28f5dd2f70854ef94cc66'


class GroestlcoinBlockTestCase(unittest.TestCase):
    def _get_block_stream(self):
        return io.BytesIO(h2b(
            "000000202c962905d18d95055e0b8d8f45ce748a44c119817058c23465b71100000000"
            "0077da86d33f0c2e51b3e2aa2ba24551141edd68a1cc890e083521361993c31932c4f7"
            "a65aecf5141baa90713202010000000001010000000000000000000000000000000000"
            "000000000000000000000000000000ffffffff4c0380841e04c4f7a65a08fabe6d6d33"
            "312d33302d383130325b6d36335b1b3691506e2a97a2c14df7e9bc6e94e54001000000"
            "0000000060000413010200000d2f6e6f64655374726174756d2f000000000200000000"
            "00000000266a24aa21a9ed8dfe2e8f6eb7ff2d6e27732d11c6aaee5eab7d5ea67b6cf8"
            "87ec9d4cd038e831a876cd1d000000001976a91442bbcfb1cd88dbbb4c56086f1d018f"
            "10a7ef760688ac01200000000000000000000000000000000000000000000000000000"
            "000000000000000000000100000001f7b346fc08b41de682b81b7ce729edb2aeddc2f5"
            "87b759859031e3bf99e68779000000006b483045022100e00b07400fbe10ee93962425"
            "57026ff6d3e16ffb46add2ccc5d7ffd8ddbc1989022003cc6832c4d62e154781507488"
            "67ca71eca04655f4b2ce31773832353d571647012102d4ad38a64b4ec115a77a900d6f"
            "b7f44343aa761ab6fcf8b8613443fa7e24e5befeffffff026e539827000000001976a9"
            "145db12e800001e767f56a3dbb202d0666315d063288acb5162400000000001976a914"
            "bd7b0274f1ef4325d0e8e273ed5be328bc3d11d988ac4a841e00"))

    def test_block_hash(self):
        parsed = GroestlcoinMainnet.block.parse(self._get_block_stream())
        self.assertEqual(str(parsed.hash()), BLOCK_HASH)

    def test_block_data(self):
        parsed = GroestlcoinMainnet.block.parse(self._get_block_stream())
        self.assertEqual(parsed.version, 536870912)
        self.assertEqual(str(parsed.previous_block_hash),
                         '000000000011b76534c258708119c1448a74ce458f8d0b5e05958dd10529962c')
        self.assertEqual(str(parsed.merkle_root),
                         '3219c39319362135080e89cca168dd1e145145a22baae2b3512e0c3fd386da77')
        self.assertEqual(parsed.timestamp, 1520891844)
        self.assertEqual(parsed.difficulty, int('1b14f5ec', 16))
        self.assertEqual(parsed.nonce, 846303402)

    def test_block_txs(self):
        parsed = GroestlcoinMainnet.block.parse(self._get_block_stream())
        tx = parsed.txs[0]
        self.assertEqual(str(tx.hash()), '7a64cd89b9e0fd6b32be5deb349e460a90e6805278dee2fe301b347623f0806b')
        self.assertTrue(tx.is_coinbase())

        tx = parsed.txs[1]
        self.assertEqual(str(tx.hash()), '89ff39e83636359bace69725fe19e1d33b16918205eaaf77c37684ac557e5154')
        self.assertEqual(tx.txs_out[0].coin_value, 664294254)
        self.assertEqual(GroestlcoinMainnet.address.for_script(tx.txs_out[0].puzzle_script()),
                         'Fdi7YYeN7nmnjJPqQDGy8vudgQ26FVjNrk')
        self.assertEqual(tx.txs_out[1].coin_value, 2365109)
        self.assertEqual(GroestlcoinMainnet.address.for_script(tx.txs_out[1].puzzle_script()),
                         'FnSbX2G6VkrCqCLBFPxwbrC9xT7C3rh5ko')



from .parseable_str import parse_b58_double_sha256, parse_bech32, parse_colon_prefix, parseable_str

from pycoin.contrib import segwit_addr
from pycoin.encoding.bytes32 import from_bytes_32
from pycoin.intbytes import int2byte
from pycoin.encoding.hexbytes import b2h, h2b

from .Contract import Contract


class ParseAPI(object):
    def __init__(
            self, network, bip32_prv_prefix=None, bip32_pub_prefix=None, address_prefix=None,
            pay_to_script_prefix=None, bech32_hrp=None, wif_prefix=None, sec_prefix=None):
        self._network = network
        self._bip32_prv_prefix = bip32_prv_prefix
        self._bip32_pub_prefix = bip32_pub_prefix
        self._address_prefix = address_prefix
        self._pay_to_script_prefix = pay_to_script_prefix
        self._bech32_hrp = bech32_hrp
        self._wif_prefix = wif_prefix
        self._sec_prefix = sec_prefix

    def parse_b58_hashed(self, s):
        """
        Override me to change how the b58 hashing check is done.
        """
        return parse_b58_double_sha256(s)

    # hierarchical key
    def bip32_seed(self, s):
        """
        Parse a bip32 private key from a seed.
        Return a :class:`BIP32 <pycoin.key.BIP32Node.BIP32Node>` or None.
        """
        pair = parse_colon_prefix(s)
        if pair is None or pair[0] not in "HP":
            return None
        if pair[0] == "H":
            try:
                master_secret = h2b(pair[1])
            except ValueError:
                return None
        else:
            master_secret = pair[1].encode("utf8")
        return self._network.keys.bip32_seed(master_secret)

    def bip32_prv(self, s):
        """
        Parse a bip32 private key from a text string ("xprv" type).
        Return a :class:`BIP32 <pycoin.key.BIP32Node.BIP32Node>` or None.
        """
        data = self.parse_b58_hashed(s)
        if data is None or not data.startswith(self._bip32_prv_prefix):
            return None
        return self._network.keys.bip32_deserialize(data)

    def bip32_pub(self, s):
        """
        Parse a bip32 public key from a text string ("xpub" type).
        Return a :class:`BIP32 <pycoin.key.BIP32Node.BIP32Node>` or None.
        """
        data = self.parse_b58_hashed(s)
        if data is None or not data.startswith(self._bip32_pub_prefix):
            return None
        return self._network.keys.bip32_deserialize(data)

    def bip32(self, s):
        """
        Parse a bip32 public key from a text string, either a seed, a prv or a pub.
        Return a :class:`BIP32 <pycoin.key.BIP32Node.BIP32Node>` or None.
        """
        s = parseable_str(s)
        return self.bip32_prv(s) or self.bip32_pub(s)

    def _electrum_to_blob(self, s):
        pair = parse_colon_prefix(s)
        if pair is None or pair[0] != "E":
            return None
        try:
            return h2b(pair[1])
        except ValueError:
            return None

    def electrum_seed(self, s):
        """
        Parse an electrum key from a text string in seed form ("E:xxx" where xxx
        is a 32-character hex string).
        Return a :class:`ElectrumWallet <pycoin.key.electrum.ElectrumWallet>` or None.
        """
        blob = self._electrum_to_blob(s)
        if blob and len(blob) == 16:
            blob = b2h(blob)
            return self._network.keys.electrum_seed(seed=blob)

    def electrum_prv(self, s):
        """
        Parse an electrum private key from a text string in seed form ("E:xxx" where xxx
        is a 64-character hex string).
        Return a :class:`ElectrumWallet <pycoin.key.electrum.ElectrumWallet>` or None.
        """
        blob = self._electrum_to_blob(s)
        if blob and len(blob) == 32:
            mpk = from_bytes_32(blob)
            return self._network.keys.electrum_private(master_private_key=mpk)

    def electrum_pub(self, s):
        """
        Parse an electrum public key from a text string in seed form ("E:xxx" where xxx
        is a 128-character hex string).
        Return a :class:`ElectrumWallet <pycoin.key.electrum.ElectrumWallet>` or None.
        """
        blob = self._electrum_to_blob(s)
        if blob and len(blob) == 64:
            return self._network.keys.electrum_public(master_public_key=blob)

    # address
    def p2pkh(self, s):
        """
        Parse a pay-to-public-key-hash address.
        Return a :class:`Contract <pycoin.networks.Contract.Contract>` or None.
        """
        data = self.parse_b58_hashed(s)
        if data is None or not data.startswith(self._address_prefix):
            return None
        size = len(self._address_prefix)
        script = self._network.contract.for_p2pkh(data[size:])
        script_info = self._network.contract.info_for_script(script)
        return Contract(script_info, self._network)

    def p2sh(self, s):
        """
        Parse a pay-to-script-hash address.
        Return a :class:`Contract <pycoin.networks.Contract.Contract>` or None.
        """
        data = self.parse_b58_hashed(s)
        if (None in (data, self._pay_to_script_prefix) or
                not data.startswith(self._pay_to_script_prefix)):
            return None
        size = len(self._pay_to_script_prefix)
        script = self._network.contract.for_p2sh(data[size:])
        script_info = self._network.contract.info_for_script(script)
        return Contract(script_info, self._network)

    def _segwit(self, s, blob_len, segwit_attr):
        script_f = getattr(self._network.contract, segwit_attr, None)
        if script_f is None:
            return None
        pair = parse_bech32(s)
        if pair is None or pair[0] != self._bech32_hrp or pair[1] is None:
            return None
        data = pair[1]
        version_byte = int2byte(data[0])
        decoded = segwit_addr.convertbits(data[1:], 5, 8, False)
        decoded_data = b''.join(int2byte(d) for d in decoded)
        if version_byte != b'\0' or len(decoded_data) != blob_len:
            return None
        script = script_f(decoded_data)
        script_info = self._network.contract.info_for_script(script)
        return Contract(script_info, self._network)

    def p2pkh_segwit(self, s):
        """
        Parse a pay-to-pubkey-hash segwit address.
        Return a :class:`Contract <pycoin.networks.Contract.Contract>` or None.
        """
        return self._segwit(s, 20, "for_p2pkh_wit")

    def p2sh_segwit(self, s):
        """
        Parse a pay-to-script-hash segwit address.
        Return a :class:`Contract <pycoin.networks.Contract.Contract>` or None.
        """
        return self._segwit(s, 32, "for_p2sh_wit")

    # payable (+ all address types)
    def script(self, s):
        """
        Parse a script by compiling it.
        Return a :class:`Contract <pycoin.networks.Contract.Contract>` or None.
        """
        try:
            script = self._network.script.compile(s)
            script_info = self._network.contract.info_for_script(script)
            return Contract(script_info, self._network)
        except Exception:
            return None

    def as_number(self, s):
        try:
            return int(s)
        except ValueError:
            pass
        try:
            return int(s, 16)
        except ValueError:
            pass

    # private key
    def wif(self, s):
        """
        Parse a WIF.
        Return a :class:`Key <pycoin.key.Key>` or None.
        """
        data = self.parse_b58_hashed(s)
        if data is None or not data.startswith(self._wif_prefix):
            return None
        data = data[len(self._wif_prefix):]
        is_compressed = (len(data) > 32)
        if is_compressed:
            data = data[:-1]
        se = from_bytes_32(data)
        return self._network.keys.private(se, is_compressed=is_compressed)

    def secret_exponent(self, s):
        """
        Parse an integer secret exponent.
        Return a :class:`Key <pycoin.key.Key>` or None.
        """
        v = self.as_number(s)
        if v:
            try:
                return self._network.keys.private(v)
            except ValueError:
                pass

    # public key
    def public_pair(self, s):
        """
        Parse a public pair X/Y or X,Y where X is a coordinate and Y is a coordinate or
        the string "even" or "odd".
        Return a :class:`Key <pycoin.key.Key>` or None.
        """
        point = None
        Key = self._network.keys.private
        # BRAIN DAMAGE
        generator = Key(1)._generator
        for c in ",/":
            if c in s:
                s0, s1 = s.split(c, 1)
                v0 = self.as_number(s0)
                if v0:
                    if s1 in ("even", "odd"):
                        is_y_odd = (s1 == "odd")
                        point = generator.points_for_x(v0)[is_y_odd]
                    v1 = self.as_number(s1)
                    if v1:
                        if generator.contains_point(v0, v1):
                            point = generator.Point(v0, v1)
        if point:
            return self._network.keys.public(point)

    def sec(self, s):
        """
        Parse a public pair as a text SEC.
        Return a :class:`Key <pycoin.key.Key>` or None.
        """
        pair = parse_colon_prefix(s)
        if pair is not None and pair[0] == self._wif_prefix:
            s = pair[1]
        try:
            sec = h2b(s)
            return self._network.keys.public(sec)
        except Exception:
            pass

    def address(self, s):
        """
        Parse an address, any of p2pkh, p2sh, p2pkh_segwit, or p2sh_segwit.
        Return a :class:`Contract <pycoin.networks.Contract.Contract>`, or None.
        """
        s = parseable_str(s)
        return self.p2pkh(s) or self.p2sh(s) or self.p2pkh_segwit(s) or self.p2sh_segwit(s)

    def payable(self, s):
        """
        Parse text as either an address or a script to be compiled.
        Return a :class:`Contract <pycoin.networks.Contract.Contract>`, or None.
        """
        s = parseable_str(s)
        return self.address(s) or self.script(s)

    # semantic items
    def hierarchical_key(self, s):
        """
        Parse text as some kind of hierarchical key.
        Return a subclass of :class:`Key <pycoin.key.Key>`, or None.
        """
        s = parseable_str(s)
        for f in [self.bip32_seed, self.bip32_prv, self.bip32_pub,
                  self.electrum_seed, self.electrum_prv, self.electrum_pub]:
            v = f(s)
            if v:
                return v

    def private_key(self, s):
        """
        Parse text as some kind of private key.
        Return a subclass of :class:`Key <pycoin.key.Key>`, or None.
        """
        s = parseable_str(s)
        for f in [self.wif, self.secret_exponent]:
            v = f(s)
            if v:
                return v

    def secret(self, s):
        """
        Parse text either a private key or a private hierarchical key.
        Return a subclass of :class:`Key <pycoin.key.Key>`, or None.
        """
        s = parseable_str(s)
        for f in [self.private_key, self.hierarchical_key]:
            v = f(s)
            if v:
                return v

    def public_key(self, s):
        """
        Parse text as either a public pair or an sec.
        Return a subclass of :class:`Key <pycoin.key.Key>`, or None.
        """
        s = parseable_str(s)
        for f in [self.public_pair, self.sec]:
            v = f(s)
            if v:
                return v

    def input(self, s):
        """
        NOT YET SUPPORTED
        """
        # BRAIN DAMAGE: TODO
        return None

    def tx(self, s):
        """
        NOT YET SUPPORTED
        """
        # BRAIN DAMAGE: TODO
        return None

    def spendable(self, s):
        """
        NOT YET SUPPORTED
        """
        # BRAIN DAMAGE: TODO
        return None

    def script_preimage(self, s):
        """
        NOT YET SUPPORTED
        """
        # BRAIN DAMAGE: TODO
        return None

    def __call__(self, s):
        s = parseable_str(s)
        return (self.payable(s) or
                self.input(s) or
                self.secret(s) or
                self.tx(s))



from .Generator import Generator
from .native.openssl import create_OpenSSLOptimizations, NID_X9_62_prime256v1


_p = 0xffffffff00000001000000000000000000000000ffffffffffffffffffffffff
_a = 0xffffffff00000001000000000000000000000000fffffffffffffffffffffffc
_b = 0x5ac635d8aa3a93e7b3ebbd55769886bc651d06b0cc53b0f63bce3c3e27d2604b
_Gx = 0x6b17d1f2e12c4247f8bce6e563a440f277037d812deb33a0f4a13945d898c296
_Gy = 0x4fe342e2fe1a7f9b8ee7eb4a7c0f9e162bce33576b315ececbb6406837bf51f5
_r = 0xffffffff00000000ffffffffffffffffbce6faada7179e84f3b9cac2fc632551


class GeneratorWithOptimizations(create_OpenSSLOptimizations(NID_X9_62_prime256v1), Generator):
    pass


secp256r1_generator = GeneratorWithOptimizations(_p, _a, _b, (_Gx, _Gy), _r)



import unittest

from pycoin.ecdsa.encrypt import generate_shared_public_key
from pycoin.ecdsa.secp256k1 import secp256k1_generator


class SharedPublicKeyTest(unittest.TestCase):
    def test_gen_shared(self):
        BASE_1 = 0x1111111111111111111111111111111111111111111111111111111111111111
        BASE_2 = 0x1111111111011111111111111110111111111111111111110111111110111111
        for factor_1 in range(1, 16):
            priv_1 = BASE_1 * factor_1
            pub_1 = secp256k1_generator * priv_1
            for factor_2 in range(1, 16):
                priv_2 = BASE_2 * factor_2
                pub_2 = secp256k1_generator * priv_2
                pk1 = generate_shared_public_key(priv_1, pub_2, secp256k1_generator)
                pk2 = generate_shared_public_key(priv_2, pub_1, secp256k1_generator)
                self.assertEqual(pk1, pk2)



import unittest

from pycoin.blockchain.ChainFinder import ChainFinder


class BHO(object):
    def __init__(self, h, previous_block_hash=None, difficulty=10):
        self.h = h
        if previous_block_hash is None:
            previous_block_hash = h-1
        self.previous_block_hash = previous_block_hash
        self.difficulty = difficulty

    def hash(self):
        return self.h

    def __repr__(self):
        return "<BHO: id:%s parent:%s difficulty:%s>" % \
            (self.h, self.previous_block_hash, self.difficulty)


def do_scramble(items, tfb, dbt):
    import itertools
    for c in itertools.permutations(items):
        cf = ChainFinder()
        load_items(cf, c)
        assert cf.trees_from_bottom == tfb
        assert cf.descendents_by_top == dbt
        cf = ChainFinder()
        for b in c:
            load_items(cf, [b])
        assert cf.trees_from_bottom == tfb
        assert cf.descendents_by_top == dbt


def load_items(cf, bhos):
    return cf.load_nodes((bh.h, bh.previous_block_hash) for bh in bhos)


class ChainFinderTestCase(unittest.TestCase):

    def test_basics(self):
        cf = ChainFinder()
        assert cf.trees_from_bottom == {}
        assert cf.descendents_by_top == {}
        ITEMS = [BHO(i) for i in range(6)]

        load_items(cf, [ITEMS[0]])
        assert cf.trees_from_bottom == {0: [0, -1]}
        assert cf.descendents_by_top == {-1: {0}}

        load_items(cf, [ITEMS[1]])
        assert cf.trees_from_bottom == {1: [1, 0, -1]}
        assert cf.descendents_by_top == {-1: {1}}

        load_items(cf, ITEMS[0:2])
        assert cf.trees_from_bottom == {1: [1, 0, -1]}
        assert cf.descendents_by_top == {-1: {1}}

        load_items(cf, [ITEMS[4]])
        assert cf.trees_from_bottom == {1: [1, 0, -1], 4: [4, 3]}
        assert cf.descendents_by_top == {-1: {1}, 3: {4}}

        load_items(cf, [ITEMS[3]])
        assert cf.trees_from_bottom == {1: [1, 0, -1], 4: [4, 3, 2]}
        assert cf.descendents_by_top == {-1: {1}, 2: {4}}

        load_items(cf, [ITEMS[5]])
        assert cf.trees_from_bottom == {1: [1, 0, -1], 5: [5, 4, 3, 2]}
        assert cf.descendents_by_top == {-1: {1}, 2: {5}}

        load_items(cf, [ITEMS[2]])
        assert cf.trees_from_bottom == {5: [5, 4, 3, 2, 1, 0, -1]}
        assert cf.descendents_by_top == {-1: {5}}

        do_scramble(ITEMS, cf.trees_from_bottom, cf.descendents_by_top)

    def test_branch(self):
        cf = ChainFinder()
        assert cf.trees_from_bottom == {}
        assert cf.descendents_by_top == {}
        ITEMS = [BHO(i) for i in range(7)]
        B301 = BHO(301, 3, 10)
        B302, B303, B304 = [BHO(i) for i in range(302, 305)]

        load_items(cf, [B302])
        assert cf.trees_from_bottom == {302: [302, 301]}
        assert cf.descendents_by_top == {301: {302}}

        load_items(cf, [B304])
        assert cf.trees_from_bottom == {302: [302, 301], 304: [304, 303]}
        assert cf.descendents_by_top == {301: {302}, 303: {304}}

        load_items(cf, [B303])
        assert cf.trees_from_bottom == {304: [304, 303, 302, 301]}
        assert cf.descendents_by_top == {301: {304}}

        load_items(cf, ITEMS)
        assert cf.trees_from_bottom == {
            6: [6, 5, 4, 3, 2, 1, 0, -1],
            304: [304, 303, 302, 301]
        }
        assert cf.descendents_by_top == {-1: {6}, 301: {304}}

        load_items(cf, [B301])
        assert cf.trees_from_bottom == {
            6: [6, 5, 4, 3, 2, 1, 0, -1],
            304: [304, 303, 302, 301, 3, 2, 1, 0, -1]
        }
        assert cf.descendents_by_top == {-1: {6, 304}}

    def test_0123(self):
        I0 = BHO(0)
        I1 = BHO(1)
        I2 = BHO(2)
        I3 = BHO(3, 1)
        cf = ChainFinder()
        load_items(cf, [I0, I2, I3, I1])
        assert cf.trees_from_bottom == {2: [2, 1, 0, -1], 3: [3, 1, 0, -1]}
        assert cf.descendents_by_top == {-1: {2, 3}}

    def test_all_orphans(self):
        I1 = BHO(1)
        I2 = BHO(2)
        I3 = BHO(3)
        cf = ChainFinder()
        load_items(cf, [I2, I3, I1])
        assert cf.trees_from_bottom == {3: [3, 2, 1, 0]}
        assert cf.descendents_by_top == {0: {3}}

    def test_scramble(self):
        ITEMS = [BHO(i, (i-1)//2, 10) for i in range(7)]
        tfb = {
            3: [3, 1, 0, -1],
            4: [4, 1, 0, -1],
            5: [5, 2, 0, -1],
            6: [6, 2, 0, -1]
        }
        dbt = {-1: {3, 4, 5, 6}}
        do_scramble(ITEMS, tfb, dbt)

    def test_branch_switch(self):
        cf = ChainFinder()
        assert cf.trees_from_bottom == {}
        assert cf.descendents_by_top == {}
        ITEMS = [BHO(i) for i in range(4)]
        B201 = BHO(201, 2, 10)
        B202, B203, B204 = [BHO(i) for i in range(202, 205)]

        items = ITEMS + [B201, B202, B203, B204]
        tfb = {204: [204, 203, 202, 201, 2, 1, 0, -1], 3: [3, 2, 1, 0, -1]}
        dbt = {-1: {3, 204}}
        do_scramble(items, tfb, dbt)

    def test_longest_chain_endpoint(self):
        cf = ChainFinder()
        ITEMS = [BHO(i) for i in range(5)]
        B201 = BHO(201, 2, 110)
        B202, B203, B204 = [BHO(i) for i in range(202, 205)]

        def node_weight_f(h):
            if h == -1:
                return 0
            if h == 201:
                return 110
            return 10

        items = ITEMS + [B201, B202, B203, B204]
        load_items(cf, items)
        # assert cf.difficulty(0, node_weight_f) == 10
        # assert cf.difficulty(1, node_weight_f) == 20
        # assert cf.difficulty(2, node_weight_f) == 30
        # assert cf.difficulty(3, node_weight_f) == 40
        # assert cf.difficulty(4, node_weight_f) == 50
        # assert cf.difficulty(201, node_weight_f) == 140
        # assert cf.difficulty(202, node_weight_f) == 150

    def test_find_ancestral_path(self):

        ITEMS = [BHO(i) for i in range(5)]
        B201 = BHO(201, 2, 110)
        B202, B203, B204 = [BHO(i) for i in range(202, 205)]

        cf = ChainFinder()
        items = ITEMS + [B202, B203, B204]
        load_items(cf, items)

        load_items(cf, [B201])

        old_chain_endpoint, new_chain_endpoint = 4, 204

        old_subpath, new_subpath = cf.find_ancestral_path(old_chain_endpoint, new_chain_endpoint)
        assert old_subpath == [4, 3, 2]
        assert new_subpath == [204, 203, 202, 201, 2]

    def test_large(self):
        ITEMS = [BHO(i) for i in range(10000)]
        cf = ChainFinder()
        load_items(cf, ITEMS)
        old_subpath, new_subpath = cf.find_ancestral_path(5000, 9000)



from pycoin.encoding.hexbytes import b2h, b2h_rev
from pycoin.coins.bitcoin.Tx import Tx


class BitcoindProvider(object):
    def __init__(self, bitcoind_url):
        try:
            from bitcoinrpc.authproxy import AuthServiceProxy
        except ImportError:
            print("This script depends upon python-bitcoinrpc.")
            print("pip install -e git+https://github.com/jgarzik/"
                  "python-bitcoinrpc#egg=python_bitcoinrpc-master")
            raise
        self.bitcoind_url = bitcoind_url
        self.connection = AuthServiceProxy(bitcoind_url)

    def bitcoind_agrees_on_transaction_validity(self, tx):
        tx.check_unspents()
        unknown_tx_outs = [unspent_to_bitcoind_dict(tx_in, tx_out)
                           for tx_in, tx_out in zip(tx.txs_in, tx.unspents)]
        signed = self.connection.signrawtransaction(tx.as_hex(), unknown_tx_outs, [])
        is_ok = [tx.is_solution_ok(idx) for idx in range(len(tx.txs_in))]
        return all(is_ok) == signed.get("complete")

    def tx_for_tx_hash(self, tx_hash):
        raw_tx = self.connection.getrawtransaction(b2h_rev(tx_hash))
        tx = Tx.from_hex(raw_tx)
        return tx


def unspent_to_bitcoind_dict(tx_in, tx_out):
    return dict(
        txid=b2h_rev(tx_in.previous_hash),
        vout=tx_in.previous_index,
        scriptPubKey=b2h(tx_out.script)
    )


def bitcoind_agrees_on_transaction_validity(bitcoind_url, tx):
    bp = BitcoindProvider(bitcoind_url)
    return bp.bitcoind_agrees_on_transaction_validity(tx)




from .agent import urlopen

from pycoin.coins.bgold.Tx import Tx
from pycoin.encoding.hexbytes import b2h_rev


class BTGExpProvider(object):
    def __init__(self):
        self.base_url = "http://btgexp.com/api/"

    def tx_for_tx_hash(self, tx_hash):
        URL = "%s/getrawtransaction?txid=%s&decrypt=0" % (self.base_url, b2h_rev(tx_hash))
        r = urlopen(URL).read().decode("utf8")
        tx = Tx.from_hex(r)
        if tx.hash() == tx_hash:
            return tx
        return None



#!/usr/bin/env python

from __future__ import print_function

import argparse

from pycoin.encoding.b58 import a2b_base58, b2a_base58, a2b_hashed_base58, b2a_hashed_base58
from pycoin.encoding.hexbytes import b2h, h2b


def create_parser():
    parser = argparse.ArgumentParser(description='Convert b58 to hex or back')
    parser.add_argument('-b', help='force b58 input (rather than best guess)', action="store_true")
    parser.add_argument('input', nargs="+", help='hex or base58')
    return parser


def parse_arg(arg, force_b58):
    is_hex_input = False
    blob = None
    if not force_b58:
        try:
            blob = h2b(arg)
            is_hex_input = True
        except Exception:
            pass
    if blob is None:
        try:
            blob = a2b_base58(arg)
        except KeyError:
            pass
    if blob is None:
        raise argparse.ArgumentTypeError("can't parse %s" % arg)
    return blob, is_hex_input


def b58(args, parser):
    for arg in args.input:
        blob, is_hex_input = parse_arg(arg, args.b)

        if is_hex_input:
            print(b2h(blob))
            print(b2a_base58(blob))
            print(b2a_hashed_base58(blob))
        else:
            print(b2h(blob))
            print(b2a_base58(blob))
            try:
                blob = a2b_hashed_base58(arg)
                print("valid hashed b58")
                print("contents: ", b2h(blob))
            except Exception:
                print("not hashed b58")


def main():
    parser = create_parser()
    args = parser.parse_args()
    b58(args, parser)


if __name__ == '__main__':
    main()



from pycoin.encoding.hash import hash160
from pycoin.encoding.hexbytes import b2h, b2h_rev, h2b
from pycoin.satoshi.satoshi_struct import parse_struct, stream_struct

from .ScriptTools import BitcoinScriptTools as ScriptTools  # BRAIN DAMAGE


ZERO = b'\0' * 32


class TxIn(object):
    """
    The part of a Tx that specifies where the Bitcoin comes from.
    """

    def __init__(self, previous_hash, previous_index, script=b'', sequence=4294967295):
        self.previous_hash = previous_hash
        self.previous_index = previous_index
        self.script = script
        self.sequence = sequence
        self.witness = ()

    @classmethod
    def coinbase_tx_in(class_, script):
        tx = class_(previous_hash=ZERO, previous_index=4294967295, script=script)
        return tx

    def stream(self, f, blank_solutions=False):
        script = b'' if blank_solutions else self.script
        stream_struct("#LSL", f, self.previous_hash, self.previous_index, script, self.sequence)

    @classmethod
    def parse(self, f):
        return self(*parse_struct("#LSL", f))

    def is_coinbase(self):
        return self.previous_hash == ZERO

    def public_key_sec(self):
        """Return the public key as sec, or None in case of failure."""
        if self.is_coinbase():
            return None
        opcodes = ScriptTools.opcode_list(self.script)
        if len(opcodes) == 2 and opcodes[0].startswith("[30"):
            # the second opcode is probably the public key as sec
            sec = h2b(opcodes[1][1:-1])
            return sec
        return None

    def address(self, address_api):
        if self.is_coinbase():
            return "(coinbase)"
        # attempt to return the source address
        sec = self.public_key_sec()
        if sec:
            address = address_api.for_p2pkh(hash160(sec))
            return address
        return "(unknown)"

    def __str__(self):
        if self.is_coinbase():
            return 'TxIn<COINBASE: %s>' % b2h(self.script)
        return 'TxIn<%s[%d] "%s">' % (
            b2h_rev(self.previous_hash), self.previous_index, ScriptTools.disassemble(self.script))


"""
The MIT License (MIT)

Copyright (c) 2013 by Richard Kiss

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""
